package su.a71.new_soviet.blocks;

import net.minecraft.block.*;
import net.minecraft.block.enums.WallMountLocation;
import net.minecraft.entity.Entity;
import net.minecraft.fluid.FluidState;
import net.minecraft.fluid.Fluids;
import net.minecraft.item.ItemPlacementContext;
import net.minecraft.state.StateManager;
import net.minecraft.state.property.BooleanProperty;
import net.minecraft.state.property.EnumProperty;
import net.minecraft.state.property.Properties;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Direction;
import net.minecraft.util.shape.VoxelShape;
import net.minecraft.world.BlockView;
import net.minecraft.world.World;
import net.minecraft.world.WorldAccess;
import org.jetbrains.annotations.Nullable;
import su.a71.new_soviet.util.Shapes;

import java.util.List;

public class ConcreteWithBarsBlock extends WallMountedBlock implements Waterloggable {
    public static final BooleanProperty WATERLOGGED;
    public static final Shapes.HorizontalShape SHAPE_WALL;
    public static final Shapes.HorizontalShape SHAPE_FLOOR;
    public static final Shapes.HorizontalShape SHAPE_CEILING;
    public static final EnumProperty<WallMountLocation> FACE;

    public ConcreteWithBarsBlock(Settings settings) {
        super(settings);
        setDefaultState(getDefaultState()
                .with(Properties.HORIZONTAL_FACING, Direction.NORTH)
                .with(FACE, WallMountLocation.FLOOR)
                .with(WATERLOGGED, false));
    }

    @Override
    protected void appendProperties(StateManager.Builder<Block, BlockState> builder) {
        builder.add(Properties.HORIZONTAL_FACING, FACE, WATERLOGGED);
    }

    @Nullable
    public BlockState getPlacementState(ItemPlacementContext ctx) {
        FluidState fluidState = ctx.getWorld().getFluidState(ctx.getBlockPos());

        for (Direction direction : ctx.getPlacementDirections()) {
            BlockState blockState;
            if (direction.getAxis() == Direction.Axis.Y) {
                blockState = this.getDefaultState().with(FACE, direction == Direction.UP ? WallMountLocation.CEILING : WallMountLocation.FLOOR).with(FACING, ctx.getHorizontalPlayerFacing());
            } else {
                blockState = this.getDefaultState().with(FACE, WallMountLocation.WALL).with(FACING, direction.getOpposite());
            }

            if (blockState.canPlaceAt(ctx.getWorld(), ctx.getBlockPos())) {
                return blockState.with(WATERLOGGED, fluidState.getFluid() == Fluids.WATER);
            }
        }

        return null;
    }

    public void onLandedUpon(World world, BlockState state, BlockPos pos, Entity entity, float fallDistance) {
        if (state.get(FACE) == WallMountLocation.CEILING) {
            entity.handleFallDamage(fallDistance + 2.0F, 2.0F, world.getDamageSources().stalagmite());
        } else {
            super.onLandedUpon(world, state, pos, entity, fallDistance);
        }
    }

    public VoxelShape getOutlineShape(BlockState state, BlockView world, BlockPos pos, ShapeContext context) {
        return switch (state.get(FACE)) {
            case FLOOR -> SHAPE_FLOOR.north();
            case WALL -> switch (state.get(FACING)) {
                case NORTH -> SHAPE_WALL.north();
                case SOUTH -> SHAPE_WALL.south();
                case WEST -> SHAPE_WALL.west();
                default -> SHAPE_WALL.east();
            };
            default -> SHAPE_CEILING.north();
        };
    }

    @Override
    public BlockState getStateForNeighborUpdate(BlockState state, Direction direction, BlockState neighborState, WorldAccess world, BlockPos pos, BlockPos neighborPos) {
        if (state.get(WATERLOGGED)) {
            world.scheduleFluidTick(pos, Fluids.WATER, Fluids.WATER.getTickRate(world));
        }
        return super.getStateForNeighborUpdate(state, direction, neighborState, world, pos, neighborPos);
    }

    public FluidState getFluidState(BlockState state) {
        return state.get(WATERLOGGED) ? Fluids.WATER.getStill(false) : super.getFluidState(state);
    }

    static {
        FACE = Properties.WALL_MOUNT_LOCATION;
        WATERLOGGED = Properties.WATERLOGGED;
        SHAPE_FLOOR = new Shapes.HorizontalShape(List.of(List.of(0.01, 0.0, 0.01, 15.99, 8.0, 15.99)));
        SHAPE_CEILING = new Shapes.HorizontalShape(List.of(List.of(0.01, 8.0, 0.01, 15.99, 15.99, 15.99)));
        SHAPE_WALL = new Shapes.HorizontalShape(List.of(List.of(0.01, 0.01, 8.0, 15.99, 15.99, 15.99)));
    }
}
