package su.a71.new_soviet.datagen;
// === ABANDON HOPE, YE WHO ENTER HERE ===

import com.google.common.collect.Lists;
import net.fabricmc.fabric.api.datagen.v1.FabricDataOutput;
import net.fabricmc.fabric.api.datagen.v1.provider.FabricRecipeProvider;
import net.minecraft.block.*;
import net.minecraft.data.server.recipe.*;
import net.minecraft.item.ItemConvertible;
import net.minecraft.item.Items;
import net.minecraft.recipe.Ingredient;
import net.minecraft.recipe.book.RecipeCategory;
import net.minecraft.registry.Registries;
import net.minecraft.registry.tag.BlockTags;
import net.minecraft.registry.tag.ItemTags;
import net.minecraft.util.Util;
import su.a71.new_soviet.registration.NSE_Blocks;
import su.a71.new_soviet.registration.NSE_Custom;
import su.a71.new_soviet.registration.NSE_Items;

import java.util.List;
import java.util.function.Consumer;

public class RecipeGenerator extends FabricRecipeProvider {
    public RecipeGenerator(FabricDataOutput generator) {
        super(generator);
    }

    private void mossRecipe(Consumer<RecipeJsonProvider> exporter, ItemConvertible output, ItemConvertible input) {
        var path = convertBetween(output, input);
        ShapelessRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, output).input(input).input(Items.VINE).criterion(FabricRecipeProvider.hasItem(output),
                FabricRecipeProvider.conditionsFromItem(output)).criterion(FabricRecipeProvider.hasItem(input),
                FabricRecipeProvider.conditionsFromItem(output)).criterion(FabricRecipeProvider.hasItem(Items.VINE),
                FabricRecipeProvider.conditionsFromItem(output)).offerTo(exporter, path + "_vine");

        ShapelessRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, output).input(input).input(Items.MOSS_BLOCK).criterion(FabricRecipeProvider.hasItem(output),
                FabricRecipeProvider.conditionsFromItem(output)).criterion(FabricRecipeProvider.hasItem(input),
                FabricRecipeProvider.conditionsFromItem(output)).criterion(FabricRecipeProvider.hasItem(Items.MOSS_BLOCK),
                FabricRecipeProvider.conditionsFromItem(output)).offerTo(exporter, path + "_moss");
    }

    private void dirtyRecipe(Consumer<RecipeJsonProvider> exporter, ItemConvertible output, ItemConvertible input) {
        var path = convertBetween(output, input);
        ShapelessRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, output).input(input).input(Items.DIRT).criterion(FabricRecipeProvider.hasItem(output),
                FabricRecipeProvider.conditionsFromItem(output)).criterion(FabricRecipeProvider.hasItem(input),
                FabricRecipeProvider.conditionsFromItem(output)).criterion(FabricRecipeProvider.hasItem(Items.DIRT),
                FabricRecipeProvider.conditionsFromItem(output)).offerTo(exporter, path + "_dirt");

        ShapelessRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, output).input(input).input(Items.GLOW_LICHEN).criterion(FabricRecipeProvider.hasItem(output),
                FabricRecipeProvider.conditionsFromItem(output)).criterion(FabricRecipeProvider.hasItem(input),
                FabricRecipeProvider.conditionsFromItem(output)).criterion(FabricRecipeProvider.hasItem(Items.GLOW_LICHEN),
                FabricRecipeProvider.conditionsFromItem(output)).offerTo(exporter, path + "_lichen");
    }

    private void concreteWithBarsRecipe(Consumer<RecipeJsonProvider> exporter, ItemConvertible output, ItemConvertible input) {
        ShapelessRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, output).input(input).input(Blocks.IRON_BARS).criterion(FabricRecipeProvider.hasItem(output),
                FabricRecipeProvider.conditionsFromItem(output)).criterion(FabricRecipeProvider.hasItem(input),
                FabricRecipeProvider.conditionsFromItem(output)).criterion(FabricRecipeProvider.hasItem(Blocks.IRON_BARS),
                FabricRecipeProvider.conditionsFromItem(output)).offerTo(exporter);
    }

    private void stoneCuttingCategory(Consumer<RecipeJsonProvider> exporter, ItemConvertible base, List<ItemConvertible> category, RecipeCategory recipe_category) {
        for (ItemConvertible itemConvertible : category) {
            offerStonecuttingRecipe(exporter, recipe_category, itemConvertible, base);
        }
    }

    private void diceRecipe(Consumer<RecipeJsonProvider> exporter, ItemConvertible dice) {
        // why can't we use BlockTags.WOODEN_BUTTONS
        var buttons = Util.make(Lists.newArrayList(), list -> {
            list.add(Blocks.OAK_BUTTON);
            list.add(Blocks.BAMBOO_BUTTON);
            list.add(Blocks.BIRCH_BUTTON);
            list.add(Blocks.CHERRY_BUTTON);
            list.add(Blocks.ACACIA_BUTTON);
            list.add(Blocks.CRIMSON_BUTTON);
            list.add(Blocks.DARK_OAK_BUTTON);
            list.add(Blocks.JUNGLE_BUTTON);
            list.add(Blocks.MANGROVE_BUTTON);
            list.add(Blocks.SPRUCE_BUTTON);
            list.add(Blocks.WARPED_BUTTON);
        });
        for (Object button : buttons) {
            offerStonecuttingRecipe(exporter, RecipeCategory.MISC, dice, (ItemConvertible) button);
        }
    }

    private void warningStripeRecipe(Consumer<RecipeJsonProvider> exporter, ItemConvertible output, ItemConvertible dye) {
        ShapedRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, output, 4)
                .pattern("D#D")
                .pattern("#D#")
                .pattern("D#D")
                .input('D', dye)
                .input('#', Items.IRON_INGOT)
                .criterion(RecipeProvider.hasItem(Items.IRON_INGOT),
                        RecipeProvider.conditionsFromItem(Items.IRON_INGOT))
                .criterion(RecipeProvider.hasItem(dye),
                        RecipeProvider.conditionsFromItem(dye))
                .offerTo(exporter);
    }

    private void crissCrossRecipe(Consumer<RecipeJsonProvider> exporter, ItemConvertible output, ItemConvertible input1, ItemConvertible input2) {
        ShapedRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, output, 4)
                .pattern("XY")
                .pattern("YX")
                .input('X', input1)
                .input('Y', input2)
                .criterion(RecipeProvider.hasItem(input1),
                        RecipeProvider.conditionsFromItem(input1))
                .criterion(RecipeProvider.hasItem(input2),
                        RecipeProvider.conditionsFromItem(input2))
                .offerTo(exporter);
    }

    private void centerRecipe(Consumer<RecipeJsonProvider> exporter, ItemConvertible output, ItemConvertible input_outer, ItemConvertible input_center) {
        ShapedRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, output, 8)
                .input('X', input_outer).input('Y', input_center)
                .pattern("XXX")
                .pattern("XYX")
                .pattern("XXX")
                .criterion(hasItem(input_outer), conditionsFromItem(input_outer))
                .criterion(hasItem(input_center), conditionsFromItem(input_center))
                .offerTo(exporter);
    }

    private void concreteRecipe(Consumer<RecipeJsonProvider> exporter, ItemConvertible output, ItemConvertible dye) {
        if (dye == null) {  // Base concrete gives 2 more but requires an extra iron bar
            ShapedRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, output, 8)
                    .input('B', Blocks.IRON_BARS).input('S', Blocks.STONE)
                    .pattern("SBS")
                    .pattern(" B ")
                    .pattern("SBS")
                    .criterion(hasItem(Blocks.IRON_BARS), conditionsFromItem(Blocks.IRON_BARS))
                    .criterion(hasItem(Blocks.STONE), conditionsFromItem(Blocks.STONE))
                    .offerTo(exporter);
            return;
        }
        ShapedRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, output, 6)
                .input('D', dye).input('B', Blocks.IRON_BARS).input('S', Blocks.STONE)
                .pattern("SBS")
                .pattern(" D ")
                .pattern("SBS")
                .criterion(hasItem(dye), conditionsFromItem(dye))
                .criterion(hasItem(Blocks.IRON_BARS), conditionsFromItem(Blocks.IRON_BARS))
                .criterion(hasItem(Blocks.STONE), conditionsFromItem(Blocks.STONE))
                .offerTo(exporter);
    }

    private void wallpaperRecipe(Consumer<RecipeJsonProvider> exporter, ItemConvertible output, ItemConvertible dye) {
        ShapedRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, output, 2)
                .input('D', dye).input('P', Items.PAPER).input('S', Items.SLIME_BALL)
                .pattern(" D ")
                .pattern("PPP")
                .pattern(" S ")
                .criterion(hasItem(dye), conditionsFromItem(dye))
                .criterion(hasItem(Items.PAPER), conditionsFromItem(Items.PAPER))
                .criterion(hasItem(Items.SLIME_BALL), conditionsFromItem(Items.SLIME_BALL))
                .offerTo(exporter);
    }

    private void linoleumRecipe(Consumer<RecipeJsonProvider> exporter, ItemConvertible output, ItemConvertible dye) {
        ShapedRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, output, 2)
                .input('D', dye).input('P', Items.PAPER).input('C', Items.CLAY).input('S', Items.SLIME_BALL)
                .pattern(" D ")
                .pattern("CPC")
                .pattern("SPS")
                .criterion(hasItem(dye), conditionsFromItem(dye))
                .criterion(hasItem(Items.PAPER), conditionsFromItem(Items.PAPER))
                .criterion(hasItem(Items.SLIME_BALL), conditionsFromItem(Items.SLIME_BALL))
                .criterion(hasItem(Items.CLAY), conditionsFromItem(Items.CLAY))
                .offerTo(exporter);
    }

    private void windowRecipe(Consumer<RecipeJsonProvider> exporter, ItemConvertible outputWindow, ItemConvertible outputPaneWindow, ItemConvertible planks_slab) {
        ShapedRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, outputWindow, 1)
                .input('W', planks_slab).input('S', Items.STICK).input('G', Blocks.GLASS_PANE)
                .pattern(" S ")
                .pattern("SGS")
                .pattern(" W ")
                .criterion(hasItem(planks_slab), conditionsFromItem(planks_slab))
                .criterion(hasItem(Blocks.GLASS_PANE), conditionsFromItem(Blocks.GLASS_PANE))
                .offerTo(exporter);
        ShapedRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, outputPaneWindow, 1)
                .input('W', planks_slab).input('S', Items.STICK).input('G', Blocks.GLASS_PANE)
                .pattern(" W ")
                .pattern("SGS")
                .pattern(" W ")
                .criterion(hasItem(planks_slab), conditionsFromItem(planks_slab))
                .criterion(hasItem(Blocks.GLASS_PANE), conditionsFromItem(Blocks.GLASS_PANE))
                .offerTo(exporter);
        ShapelessRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, outputPaneWindow).input(outputWindow).input(planks_slab).criterion(FabricRecipeProvider.hasItem(outputPaneWindow),
                FabricRecipeProvider.conditionsFromItem(outputPaneWindow)).criterion(FabricRecipeProvider.hasItem(planks_slab),
                FabricRecipeProvider.conditionsFromItem(planks_slab))
                .offerTo(exporter, Registries.ITEM.getId(outputPaneWindow.asItem()) + "_shapeless");
    }

    private void tvRecipe(Consumer<RecipeJsonProvider> exporter, ItemConvertible output, ItemConvertible dye) {
        ShapedRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, output, 1)
                .pattern("TAT")
                .pattern("IGI")
                .pattern("RDR")
                .input('D', dye)
                .input('A', NSE_Items.ANTENNA)
                .input('I', Items.IRON_INGOT)
                .input('T', Items.IRON_TRAPDOOR)
                .input('R', Items.REDSTONE)
                .input('G', Items.GLASS_PANE)
                .criterion(RecipeProvider.hasItem(Items.IRON_INGOT),
                        RecipeProvider.conditionsFromItem(Items.IRON_INGOT))
                .criterion(RecipeProvider.hasItem(Items.IRON_TRAPDOOR),
                        RecipeProvider.conditionsFromItem(Items.IRON_TRAPDOOR))
                .criterion(RecipeProvider.hasItem(NSE_Items.ANTENNA),
                        RecipeProvider.conditionsFromItem(NSE_Items.ANTENNA))
                .criterion(RecipeProvider.hasItem(Items.GLASS_PANE),
                        RecipeProvider.conditionsFromItem(Items.GLASS_PANE))
                .criterion(RecipeProvider.hasItem(Items.REDSTONE),
                        RecipeProvider.conditionsFromItem(Items.REDSTONE))
                .criterion(RecipeProvider.hasItem(dye),
                        RecipeProvider.conditionsFromItem(dye))
                .offerTo(exporter);
    }

    private void offerStairsAndSlab(Consumer<RecipeJsonProvider> exporter, ItemConvertible output_stairs, ItemConvertible output_slab, ItemConvertible input) {
        createStairsRecipe(output_stairs, Ingredient.ofItems(input))
                .criterion(hasItem(input), conditionsFromItem(input)).offerTo(exporter);
        createSlabRecipe(RecipeCategory.BUILDING_BLOCKS, output_slab, Ingredient.ofItems(input))
                .criterion(hasItem(input), conditionsFromItem(input)).offerTo(exporter);

        offerStonecuttingRecipe(exporter, RecipeCategory.BUILDING_BLOCKS, output_stairs, input);
        offerStonecuttingRecipe(exporter, RecipeCategory.BUILDING_BLOCKS, output_slab, input, 2);
    }

    @Override
    public void generate(Consumer<RecipeJsonProvider> exporter) {
        crissCrossRecipe(exporter, NSE_Blocks.SAND_TILES, Blocks.SMOOTH_SANDSTONE, Blocks.SMOOTH_RED_SANDSTONE);
        crissCrossRecipe(exporter, NSE_Blocks.LIGHT_BLUE_TILES, Blocks.STONE, Items.LIGHT_BLUE_DYE);
        crissCrossRecipe(exporter, NSE_Blocks.SMALL_WHITE_TILES, Blocks.STONE, Items.WHITE_DYE);
        crissCrossRecipe(exporter, NSE_Blocks.GREEN_WHITE_TILES, Blocks.POLISHED_DIORITE, Items.GREEN_DYE);

        tvRecipe(exporter, NSE_Custom.TV, Items.ORANGE_DYE);
        tvRecipe(exporter, NSE_Custom.BROWN_TV, Items.BROWN_DYE);
        tvRecipe(exporter, NSE_Custom.RED_TV, Items.RED_DYE);
        tvRecipe(exporter, NSE_Custom.GREEN_TV, Items.GREEN_DYE);

        windowRecipe(exporter, NSE_Blocks.ACACIA_WINDOW, NSE_Blocks.ACACIA_PANE_WINDOW, Blocks.ACACIA_SLAB);
        windowRecipe(exporter, NSE_Blocks.BIRCH_WINDOW, NSE_Blocks.BIRCH_PANE_WINDOW, Blocks.BIRCH_SLAB);
        windowRecipe(exporter, NSE_Blocks.CHERRY_WINDOW, NSE_Blocks.CHERRY_PANE_WINDOW, Blocks.CHERRY_SLAB);
        windowRecipe(exporter, NSE_Blocks.CRIMSON_WINDOW, NSE_Blocks.CRIMSON_PANE_WINDOW, Blocks.CRIMSON_SLAB);
        windowRecipe(exporter, NSE_Blocks.DARK_OAK_WINDOW, NSE_Blocks.DARK_OAK_PANE_WINDOW, Blocks.DARK_OAK_SLAB);
        windowRecipe(exporter, NSE_Blocks.JUNGLE_WINDOW, NSE_Blocks.JUNGLE_PANE_WINDOW, Blocks.JUNGLE_SLAB);
        windowRecipe(exporter, NSE_Blocks.MANGROVE_WINDOW, NSE_Blocks.MANGROVE_PANE_WINDOW, Blocks.MANGROVE_SLAB);
        windowRecipe(exporter, NSE_Blocks.OAK_WINDOW, NSE_Blocks.OAK_PANE_WINDOW, Blocks.OAK_SLAB);
        windowRecipe(exporter, NSE_Blocks.SPRUCE_WINDOW, NSE_Blocks.SPRUCE_PANE_WINDOW, Blocks.SPRUCE_SLAB);
        windowRecipe(exporter, NSE_Blocks.WARPED_WINDOW, NSE_Blocks.WARPED_PANE_WINDOW, Blocks.WARPED_SLAB);

        // Cracked blocks
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_BRICK_TILES, NSE_Blocks.BRICK_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_GLAZED_BRICK_TILES, NSE_Blocks.GLAZED_BRICK_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_DARK_BRICK_TILES, NSE_Blocks.DARK_BRICK_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_NII_WALL_1, NSE_Blocks.NII_WALL_1);
        offerCrackingRecipe(exporter, NSE_Blocks.SMALL_CRACKED_YELLOW_TILES, NSE_Blocks.SMALL_YELLOW_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_SAND_TILES, NSE_Blocks.SAND_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.SMALL_CRACKED_SAND_TILES, NSE_Blocks.SMALL_SAND_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_SAND_BRICKS, NSE_Blocks.SAND_BRICKS);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_TEAL_TILES, NSE_Blocks.TEAL_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.SMALL_CRACKED_TEAL_TILES, NSE_Blocks.SMALL_TEAL_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_GLAZED_TEAL_TILES, NSE_Blocks.GLAZED_TEAL_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_AQUAMARINE_TILES, NSE_Blocks.AQUAMARINE_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.SMALL_CRACKED_AQUAMARINE_TILES, NSE_Blocks.SMALL_AQUAMARINE_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_GLAZED_AQUAMARINE_TILES, NSE_Blocks.GLAZED_AQUAMARINE_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.SMALL_CRACKED_DIORITE_TILES, NSE_Blocks.SMALL_DIORITE_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_DIORITE_BRICKS, NSE_Blocks.DIORITE_BRICKS);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_CALCITE_TILES, NSE_Blocks.CALCITE_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_DRIPSTONE_TILES, NSE_Blocks.DRIPSTONE_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_DEEPSLATE_TILES, NSE_Blocks.DEEPSLATE_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.SMALL_CRACKED_DEEPSLATE_TILES, NSE_Blocks.SMALL_DEEPSLATE_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_LIGHT_BLUE_TILES, NSE_Blocks.LIGHT_BLUE_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.SMALL_CRACKED_LIGHT_BLUE_TILES, NSE_Blocks.SMALL_LIGHT_BLUE_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_GLAZED_LIGHT_BLUE_TILES, NSE_Blocks.GLAZED_LIGHT_BLUE_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_LIGHT_BLUE_BRICKS, NSE_Blocks.LIGHT_BLUE_BRICKS);
        offerCrackingRecipe(exporter, NSE_Blocks.SMALL_CRACKED_GRANITE_TILES, NSE_Blocks.SMALL_GRANITE_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_GREEN_WHITE_TILES, NSE_Blocks.GREEN_WHITE_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_TUFF_TILES, NSE_Blocks.TUFF_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_TUFF_BRICKS, NSE_Blocks.TUFF_BRICKS);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_WHITEWASH, NSE_Blocks.WHITEWASH);
        offerCrackingRecipe(exporter, NSE_Blocks.VERY_CRACKED_WHITEWASH, NSE_Blocks.CRACKED_WHITEWASH);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_WHITE_CONCRETE, NSE_Blocks.WHITE_CONCRETE);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_BEIGE_CONCRETE, NSE_Blocks.BEIGE_CONCRETE);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_BLUE_CONCRETE, NSE_Blocks.BLUE_CONCRETE);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_RED_CONCRETE, NSE_Blocks.RED_CONCRETE);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_YELLOW_CONCRETE, NSE_Blocks.YELLOW_CONCRETE);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_GREEN_CONCRETE, NSE_Blocks.GREEN_CONCRETE);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_GREEN_BRICKS, NSE_Blocks.GREEN_BRICKS);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_GREEN_BRICKS_MIXED, NSE_Blocks.GREEN_BRICKS_MIXED);
        offerCrackingRecipe(exporter, NSE_Blocks.BIG_GREEN_TILES_CRACKED, NSE_Blocks.BIG_GREEN_TILES);
        offerCrackingRecipe(exporter, NSE_Blocks.CRACKED_RED_BRICKS, NSE_Blocks.RED_BRICKS);

        // Mossy variants
        mossRecipe(exporter, NSE_Blocks.MOSSY_SAND_TILES, NSE_Blocks.SAND_TILES);
        mossRecipe(exporter, NSE_Blocks.MOSSY_SAND_BRICKS, NSE_Blocks.SAND_BRICKS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_TEAL_TILES, NSE_Blocks.TEAL_TILES);
        mossRecipe(exporter, NSE_Blocks.MOSSY_AQUAMARINE_TILES, NSE_Blocks.AQUAMARINE_TILES);
        mossRecipe(exporter, NSE_Blocks.MOSSY_DIORITE_BRICKS, NSE_Blocks.DIORITE_BRICKS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_CALCITE_TILES, NSE_Blocks.CALCITE_TILES);
        mossRecipe(exporter, NSE_Blocks.MOSSY_DRIPSTONE_TILES, NSE_Blocks.DRIPSTONE_TILES);
        mossRecipe(exporter, NSE_Blocks.MOSSY_DEEPSLATE_TILES, NSE_Blocks.DEEPSLATE_TILES);
        mossRecipe(exporter, NSE_Blocks.MOSSY_LIGHT_BLUE_TILES, NSE_Blocks.LIGHT_BLUE_TILES);
        mossRecipe(exporter, NSE_Blocks.MOSSY_LIGHT_BLUE_BRICKS, NSE_Blocks.LIGHT_BLUE_BRICKS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_GREEN_WHITE_TILES, NSE_Blocks.GREEN_WHITE_TILES);
        mossRecipe(exporter, NSE_Blocks.MOSSY_TUFF_TILES, NSE_Blocks.TUFF_TILES);
        mossRecipe(exporter, NSE_Blocks.MOSSY_TUFF_BRICKS, NSE_Blocks.TUFF_BRICKS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_GREEN_BRICKS, NSE_Blocks.GREEN_BRICKS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_GREEN_BRICKS_MIXED, NSE_Blocks.GREEN_BRICKS_MIXED);
        mossRecipe(exporter, NSE_Blocks.MOSSY_RED_BRICKS, NSE_Blocks.RED_BRICKS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_BRICK_TILES, NSE_Blocks.BRICK_TILES);
        mossRecipe(exporter, NSE_Blocks.MOSSY_DARK_BRICK_TILES, NSE_Blocks.DARK_BRICK_TILES);
        mossRecipe(exporter, NSE_Blocks.MOSSY_SAND_TILES_STAIRS, NSE_Blocks.SAND_TILES_STAIRS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_SAND_TILES_SLAB, NSE_Blocks.SAND_TILES_SLAB);
        mossRecipe(exporter, NSE_Blocks.MOSSY_SAND_BRICKS_STAIRS, NSE_Blocks.SAND_BRICKS_STAIRS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_SAND_BRICKS_SLAB, NSE_Blocks.SAND_BRICKS_SLAB);
        mossRecipe(exporter, NSE_Blocks.MOSSY_BRICK_TILES_STAIRS, NSE_Blocks.BRICK_TILES_STAIRS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_BRICK_TILES_SLAB, NSE_Blocks.BRICK_TILES_SLAB);
        mossRecipe(exporter, NSE_Blocks.MOSSY_DARK_BRICK_TILES_STAIRS, NSE_Blocks.DARK_BRICK_TILES_STAIRS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_DARK_BRICK_TILES_SLAB, NSE_Blocks.DARK_BRICK_TILES_SLAB);
        mossRecipe(exporter, NSE_Blocks.MOSSY_RED_BRICKS_STAIRS, NSE_Blocks.RED_BRICKS_STAIRS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_RED_BRICKS_SLAB, NSE_Blocks.RED_BRICKS_SLAB);
        mossRecipe(exporter, NSE_Blocks.MOSSY_TEAL_TILES_STAIRS, NSE_Blocks.TEAL_TILES_STAIRS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_TEAL_TILES_SLAB, NSE_Blocks.TEAL_TILES_SLAB);
        mossRecipe(exporter, NSE_Blocks.MOSSY_GREEN_BRICKS_STAIRS, NSE_Blocks.GREEN_BRICKS_STAIRS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_GREEN_BRICKS_SLAB, NSE_Blocks.GREEN_BRICKS_SLAB);
        mossRecipe(exporter, NSE_Blocks.MOSSY_GREEN_BRICKS_MIXED_STAIRS, NSE_Blocks.GREEN_BRICKS_MIXED_STAIRS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_GREEN_BRICKS_MIXED_SLAB, NSE_Blocks.GREEN_BRICKS_MIXED_SLAB);
        mossRecipe(exporter, NSE_Blocks.MOSSY_AQUAMARINE_TILES_STAIRS, NSE_Blocks.AQUAMARINE_TILES_STAIRS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_AQUAMARINE_TILES_SLAB, NSE_Blocks.AQUAMARINE_TILES_SLAB);
        mossRecipe(exporter, NSE_Blocks.MOSSY_DIORITE_BRICKS_STAIRS, NSE_Blocks.DIORITE_BRICKS_STAIRS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_DIORITE_BRICKS_SLAB, NSE_Blocks.DIORITE_BRICKS_SLAB);
        mossRecipe(exporter, NSE_Blocks.MOSSY_CALCITE_TILES_STAIRS, NSE_Blocks.CALCITE_TILES_STAIRS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_CALCITE_TILES_SLAB, NSE_Blocks.CALCITE_TILES_SLAB);
        mossRecipe(exporter, NSE_Blocks.MOSSY_DRIPSTONE_TILES_STAIRS, NSE_Blocks.DRIPSTONE_TILES_STAIRS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_DRIPSTONE_TILES_SLAB, NSE_Blocks.DRIPSTONE_TILES_SLAB);
        mossRecipe(exporter, NSE_Blocks.MOSSY_DEEPSLATE_TILES_STAIRS, NSE_Blocks.DEEPSLATE_TILES_STAIRS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_DEEPSLATE_TILES_SLAB, NSE_Blocks.DEEPSLATE_TILES_SLAB);
        mossRecipe(exporter, NSE_Blocks.MOSSY_LIGHT_BLUE_TILES_STAIRS, NSE_Blocks.LIGHT_BLUE_TILES_STAIRS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_LIGHT_BLUE_TILES_SLAB, NSE_Blocks.LIGHT_BLUE_TILES_SLAB);
        mossRecipe(exporter, NSE_Blocks.MOSSY_LIGHT_BLUE_BRICKS_STAIRS, NSE_Blocks.LIGHT_BLUE_BRICKS_STAIRS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_LIGHT_BLUE_BRICKS_SLAB, NSE_Blocks.LIGHT_BLUE_BRICKS_SLAB);
        mossRecipe(exporter, NSE_Blocks.MOSSY_GREEN_WHITE_TILES_STAIRS, NSE_Blocks.GREEN_WHITE_TILES_STAIRS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_GREEN_WHITE_TILES_SLAB, NSE_Blocks.GREEN_WHITE_TILES_SLAB);
        mossRecipe(exporter, NSE_Blocks.MOSSY_TUFF_TILES_STAIRS, NSE_Blocks.TUFF_TILES_STAIRS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_TUFF_TILES_SLAB, NSE_Blocks.TUFF_TILES_SLAB);
        mossRecipe(exporter, NSE_Blocks.MOSSY_TUFF_BRICKS_STAIRS, NSE_Blocks.TUFF_BRICKS_STAIRS);
        mossRecipe(exporter, NSE_Blocks.MOSSY_TUFF_BRICKS_SLAB, NSE_Blocks.TUFF_BRICKS_SLAB);

        // Dirty blocks recipe
        dirtyRecipe(exporter, NSE_Blocks.DIRTY_AQUAMARINE_TILES, NSE_Blocks.AQUAMARINE_TILES);
        dirtyRecipe(exporter, NSE_Blocks.DIRTY_AQUAMARINE_TILES_SLAB, NSE_Blocks.AQUAMARINE_TILES_SLAB);
        dirtyRecipe(exporter, NSE_Blocks.DIRTY_AQUAMARINE_TILES_STAIRS, NSE_Blocks.AQUAMARINE_TILES_STAIRS);
        dirtyRecipe(exporter, NSE_Blocks.DIRTY_BRICK_TILES, NSE_Blocks.BRICK_TILES);
        dirtyRecipe(exporter, NSE_Blocks.DIRTY_BRICK_TILES_SLAB, NSE_Blocks.BRICK_TILES_SLAB);
        dirtyRecipe(exporter, NSE_Blocks.DIRTY_BRICK_TILES_STAIRS, NSE_Blocks.BRICK_TILES_STAIRS);
        dirtyRecipe(exporter, NSE_Blocks.DIRTY_DARK_BRICK_TILES, NSE_Blocks.DARK_BRICK_TILES);
        dirtyRecipe(exporter, NSE_Blocks.DIRTY_DARK_BRICK_TILES_SLAB, NSE_Blocks.DARK_BRICK_TILES_SLAB);
        dirtyRecipe(exporter, NSE_Blocks.DIRTY_DARK_BRICK_TILES_STAIRS, NSE_Blocks.DARK_BRICK_TILES_STAIRS);

        // Warning stripes
        warningStripeRecipe(exporter, NSE_Blocks.INDUSTRIAL_WARNING, Items.BLACK_DYE);
        warningStripeRecipe(exporter, NSE_Blocks.RED_WARNING, Items.RED_DYE);
        warningStripeRecipe(exporter, NSE_Blocks.BLUE_WARNING, Items.BLUE_DYE);
        warningStripeRecipe(exporter, NSE_Blocks.LIGHT_BLUE_WARNING, Items.LIGHT_BLUE_DYE);
        warningStripeRecipe(exporter, NSE_Blocks.CYAN_WARNING, Items.CYAN_DYE);
        warningStripeRecipe(exporter, NSE_Blocks.YELLOW_WARNING, Items.YELLOW_DYE);
        warningStripeRecipe(exporter, NSE_Blocks.ORANGE_WARNING, Items.ORANGE_DYE);
        warningStripeRecipe(exporter, NSE_Blocks.GREEN_WARNING, Items.GREEN_DYE);
        warningStripeRecipe(exporter, NSE_Blocks.LIME_WARNING, Items.LIME_DYE);
        warningStripeRecipe(exporter, NSE_Blocks.PURPLE_WARNING, Items.PURPLE_DYE);
        warningStripeRecipe(exporter, NSE_Blocks.MAGENTA_WARNING, Items.MAGENTA_DYE);
        warningStripeRecipe(exporter, NSE_Blocks.GRAY_WARNING, Items.GRAY_DYE);

        stoneCuttingCategory(exporter, NSE_Blocks.SAND_TILES, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.SAND_BRICKS);
            list.add(NSE_Blocks.SMALL_SAND_TILES);
            list.add(NSE_Blocks.BIG_SAND_TILES);
            list.add(NSE_Blocks.BIG_SAND_BRICKS);
            list.add(NSE_Blocks.CROSS_SAND_TILES);
            list.add(NSE_Blocks.HERRINGBONE_SAND_TILES);
        }), RecipeCategory.BUILDING_BLOCKS);

        stoneCuttingCategory(exporter, Blocks.PRISMARINE_BRICKS, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.TEAL_TILES);
            list.add(NSE_Blocks.SMALL_TEAL_TILES);
            list.add(NSE_Blocks.GLAZED_TEAL_TILES);
            list.add(NSE_Blocks.VARIATED_TEAL_TILES);
        }), RecipeCategory.BUILDING_BLOCKS);
        stoneCuttingCategory(exporter, Blocks.DARK_PRISMARINE, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.AQUAMARINE_TILES);
            list.add(NSE_Blocks.GLAZED_AQUAMARINE_TILES);
            list.add(NSE_Blocks.SMALL_AQUAMARINE_TILES);
        }), RecipeCategory.BUILDING_BLOCKS);
        stoneCuttingCategory(exporter, Blocks.POLISHED_DIORITE, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.DIORITE_BRICKS);
            list.add(NSE_Blocks.BIG_DIORITE_TILES);
            list.add(NSE_Blocks.SMALL_DIORITE_TILES);
            list.add(NSE_Blocks.NII_WALL_1);
        }), RecipeCategory.BUILDING_BLOCKS);
        stoneCuttingCategory(exporter, Blocks.CALCITE, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.CALCITE_TILES);
            list.add(NSE_Blocks.DIAGONAL_CALCITE_TILES);
        }), RecipeCategory.BUILDING_BLOCKS);
        stoneCuttingCategory(exporter, Blocks.DRIPSTONE_BLOCK, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.DRIPSTONE_TILES);
            list.add(NSE_Blocks.DRIPSTONE_BRICKS);
        }), RecipeCategory.BUILDING_BLOCKS);
        stoneCuttingCategory(exporter, Blocks.POLISHED_DEEPSLATE, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.DEEPSLATE_TILES);
            list.add(NSE_Blocks.DIAGONAL_DEEPSLATE_TILES);
            list.add(NSE_Blocks.SMALL_DEEPSLATE_TILES);
        }), RecipeCategory.BUILDING_BLOCKS);
        stoneCuttingCategory(exporter, NSE_Blocks.LIGHT_BLUE_TILES, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.LIGHT_BLUE_BRICKS);
            list.add(NSE_Blocks.VARIATED_LIGHT_BLUE_TILES);
            list.add(NSE_Blocks.GLAZED_LIGHT_BLUE_TILES);
            list.add(NSE_Blocks.SMALL_LIGHT_BLUE_TILES);
        }), RecipeCategory.BUILDING_BLOCKS);
        stoneCuttingCategory(exporter, Blocks.POLISHED_GRANITE, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.BIG_GRANITE_TILES);
            list.add(NSE_Blocks.SMALL_GRANITE_TILES);
        }), RecipeCategory.BUILDING_BLOCKS);
        stoneCuttingCategory(exporter, Blocks.TUFF, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.TUFF_TILES);
            list.add(NSE_Blocks.TUFF_BRICKS);
            list.add(NSE_Blocks.BIG_TUFF_TILES);
        }), RecipeCategory.BUILDING_BLOCKS);
        stoneCuttingCategory(exporter, NSE_Blocks.SMALL_WHITE_TILES, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.SMALL_CRACKED_WHITE_TILES);
            list.add(NSE_Blocks.GLAZED_WHITE_TILES);
            list.add(NSE_Blocks.CRACKED_GLAZED_WHITE_TILES);
        }), RecipeCategory.BUILDING_BLOCKS);
        stoneCuttingCategory(exporter, Blocks.ACACIA_PLANKS, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.CROSS_ACACIA_PLANKS);
            list.add(NSE_Blocks.HERRINGBONE_ACACIA_PLANKS);
        }), RecipeCategory.BUILDING_BLOCKS);
        stoneCuttingCategory(exporter, Blocks.OAK_PLANKS, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.CROSS_OAK_PLANKS);
            list.add(NSE_Blocks.HERRINGBONE_OAK_PLANKS);
            list.add(NSE_Blocks.STRAIGHT_OAK_PARQUET);
            list.add(NSE_Blocks.SEPARATED_OAK_PARQUET);
        }), RecipeCategory.BUILDING_BLOCKS);
        stoneCuttingCategory(exporter, Blocks.BIRCH_PLANKS, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.CROSS_BIRCH_PLANKS);
            list.add(NSE_Blocks.HERRINGBONE_BIRCH_PLANKS);
        }), RecipeCategory.BUILDING_BLOCKS);
        stoneCuttingCategory(exporter, Blocks.CRIMSON_PLANKS, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.CROSS_CRIMSON_PLANKS);
            list.add(NSE_Blocks.HERRINGBONE_CRIMSON_PLANKS);
        }), RecipeCategory.BUILDING_BLOCKS);
        stoneCuttingCategory(exporter, Blocks.WARPED_PLANKS, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.CROSS_WARPED_PLANKS);
            list.add(NSE_Blocks.HERRINGBONE_WARPED_PLANKS);
        }), RecipeCategory.BUILDING_BLOCKS);
        stoneCuttingCategory(exporter, Blocks.DARK_OAK_PLANKS, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.CROSS_DARK_OAK_PLANKS);
            list.add(NSE_Blocks.HERRINGBONE_DARK_OAK_PLANKS);
            list.add(NSE_Blocks.HERRINGBONE_DARK_OAK_PARQUET);
        }), RecipeCategory.BUILDING_BLOCKS);
        stoneCuttingCategory(exporter, Blocks.JUNGLE_PLANKS, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.CROSS_JUNGLE_PLANKS);
            list.add(NSE_Blocks.HERRINGBONE_JUNGLE_PLANKS);
        }), RecipeCategory.BUILDING_BLOCKS);
        stoneCuttingCategory(exporter, Blocks.MANGROVE_PLANKS, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.CROSS_MANGROVE_PLANKS);
            list.add(NSE_Blocks.HERRINGBONE_MANGROVE_PLANKS);
        }), RecipeCategory.BUILDING_BLOCKS);
        stoneCuttingCategory(exporter, Blocks.SPRUCE_PLANKS, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.CROSS_SPRUCE_PLANKS);
            list.add(NSE_Blocks.HERRINGBONE_SPRUCE_PLANKS);
        }), RecipeCategory.BUILDING_BLOCKS);
        stoneCuttingCategory(exporter, Blocks.CHERRY_PLANKS, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.CROSS_CHERRY_PLANKS);
            list.add(NSE_Blocks.HERRINGBONE_CHERRY_PLANKS);
        }), RecipeCategory.BUILDING_BLOCKS);
        stoneCuttingCategory(exporter, Blocks.BRICKS, Util.make(Lists.newArrayList(), list -> {
            list.add(NSE_Blocks.BRICK_TILES);
            list.add(NSE_Blocks.GLAZED_BRICK_TILES);
        }), RecipeCategory.BUILDING_BLOCKS);

        offerStonecuttingRecipe(exporter, RecipeCategory.BUILDING_BLOCKS, NSE_Blocks.HERRINGBONE_BAMBOO_PLANKS, Blocks.BAMBOO_PLANKS);
        offerStonecuttingRecipe(exporter, RecipeCategory.BUILDING_BLOCKS, NSE_Blocks.NII_WALL_3, Blocks.STONE_BRICKS);
        offerStonecuttingRecipe(exporter, RecipeCategory.BUILDING_BLOCKS, NSE_Blocks.NII_WALL_2, Blocks.SMOOTH_STONE);
        offerStonecuttingRecipe(exporter, RecipeCategory.BUILDING_BLOCKS, NSE_Blocks.VINTAGE_IRON_BARS, Blocks.IRON_BARS);
        offerStonecuttingRecipe(exporter, RecipeCategory.BUILDING_BLOCKS, NSE_Blocks.METAL_PLATING, Blocks.IRON_BLOCK);

        diceRecipe(exporter, NSE_Items.DICE_D4);
        diceRecipe(exporter, NSE_Items.DICE_D6);
        diceRecipe(exporter, NSE_Items.DICE_D20);

        offerStonecuttingRecipe(exporter, RecipeCategory.BUILDING_BLOCKS, NSE_Blocks.CHISELED_MANGROVE_DOOR, Blocks.MANGROVE_DOOR);
        offerStonecuttingRecipe(exporter, RecipeCategory.BUILDING_BLOCKS, NSE_Blocks.CHISELED_OAK_DOOR, Blocks.OAK_DOOR);
        offerStonecuttingRecipe(exporter, RecipeCategory.BUILDING_BLOCKS, NSE_Blocks.CHISELED_SPRUCE_DOOR, Blocks.SPRUCE_DOOR);
        offerStonecuttingRecipe(exporter, RecipeCategory.BUILDING_BLOCKS, NSE_Blocks.CHISELED_BIRCH_DOOR, Blocks.BIRCH_DOOR);
        offerStonecuttingRecipe(exporter, RecipeCategory.BUILDING_BLOCKS, NSE_Blocks.CHISELED_ACACIA_DOOR, Blocks.ACACIA_DOOR);

        ShapedRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Custom.RADIO_RECEIVER, 1)
                .pattern(" A ")
                .pattern("WMW")
                .pattern("IRI")
                .input('M', Blocks.NOTE_BLOCK)
                .input('A', NSE_Items.ANTENNA)
                .input('I', Items.IRON_INGOT)
                .input('R', Items.REDSTONE)
                .input('W', ItemTags.PLANKS)
                .criterion(RecipeProvider.hasItem(NSE_Items.ANTENNA),
                        RecipeProvider.conditionsFromItem(NSE_Items.ANTENNA))
                .criterion(RecipeProvider.hasItem(Items.NOTE_BLOCK),
                        RecipeProvider.conditionsFromItem(Items.NOTE_BLOCK))
                .criterion(RecipeProvider.hasItem(Items.IRON_INGOT),
                        RecipeProvider.conditionsFromItem(Items.IRON_INGOT))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Custom.CEILING_FAN, 1)
                .pattern(" I ")
                .pattern("NNN")
                .input('I', Items.IRON_INGOT)
                .input('N', Items.IRON_NUGGET)
                .criterion(RecipeProvider.hasItem(Items.IRON_NUGGET),
                        RecipeProvider.conditionsFromItem(Items.IRON_NUGGET))
                .criterion(RecipeProvider.hasItem(Items.IRON_INGOT),
                        RecipeProvider.conditionsFromItem(Items.IRON_INGOT))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Custom.TABLE_LAMP, 3)
                .pattern("WWW")
                .pattern("WTW")
                .pattern(" S ")

                .input('S', Items.STICK)
                .input('T', Items.TORCH)
                .input('W', ItemTags.WOOL)
                .criterion(RecipeProvider.hasItem(Items.TORCH),
                        RecipeProvider.conditionsFromItem(Items.TORCH))
                .criterion(RecipeProvider.hasItem(Items.STICK),
                        RecipeProvider.conditionsFromItem(Items.STICK))
                .criterion("has_wool", RecipeProvider.conditionsFromTag(ItemTags.WOOL))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Custom.VINTAGE_LAMP, 3)
                .pattern("NNN")
                .pattern("GTG")
                .pattern(" N ")

                .input('N', Items.IRON_NUGGET)
                .input('T', Items.TORCH)
                .input('G', Items.GREEN_STAINED_GLASS)
                .criterion(RecipeProvider.hasItem(Items.TORCH),
                        RecipeProvider.conditionsFromItem(Items.TORCH))
                .criterion(RecipeProvider.hasItem(Items.IRON_NUGGET),
                        RecipeProvider.conditionsFromItem(Items.IRON_NUGGET))
                .criterion(RecipeProvider.hasItem(Items.GREEN_STAINED_GLASS),
                        RecipeProvider.conditionsFromItem(Items.GREEN_STAINED_GLASS))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.TOOLS, NSE_Items.SICKLE, 1)
                .pattern(" I ")
                .pattern("  I")
                .pattern(" S ")
                .input('S', Items.STICK)
                .input('I', Items.IRON_INGOT)
                .criterion(RecipeProvider.hasItem(Items.STICK),
                        RecipeProvider.conditionsFromItem(Items.STICK))
                .criterion(RecipeProvider.hasItem(Items.IRON_INGOT),
                        RecipeProvider.conditionsFromItem(Items.IRON_INGOT))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.TOOLS, NSE_Items.RAKE, 1)
                .pattern("III")
                .pattern(" N ")
                .pattern(" S ")
                .input('S', Items.STICK)
                .input('I', Items.IRON_INGOT)
                .input('N', Items.IRON_NUGGET)
                .criterion(RecipeProvider.hasItem(Items.STICK),
                        RecipeProvider.conditionsFromItem(Items.STICK))
                .criterion(RecipeProvider.hasItem(Items.IRON_INGOT),
                        RecipeProvider.conditionsFromItem(Items.IRON_INGOT))
                .criterion(RecipeProvider.hasItem(Items.IRON_NUGGET),
                        RecipeProvider.conditionsFromItem(Items.IRON_NUGGET))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.MISC, NSE_Items.LIGHT_BULB, 1)
                .input('B', Items.GLASS_BOTTLE).input('N', Items.IRON_NUGGET).input('D', Items.GLOWSTONE_DUST)
                .pattern(" D ").pattern(" B ").pattern(" N ")
                .criterion(hasItem(Items.GLASS_BOTTLE), conditionsFromItem(Items.GLASS_BOTTLE))
                .criterion(hasItem(Items.IRON_NUGGET), conditionsFromItem(Items.IRON_NUGGET))
                .criterion(hasItem(Items.GLOWSTONE_DUST), conditionsFromItem(Items.GLOWSTONE_DUST))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.MISC, NSE_Items.ANTENNA, 1)
                .input('I', Items.IRON_INGOT).input('N', Items.IRON_NUGGET)
                .pattern("N N").pattern("N N").pattern(" I ")
                .criterion(hasItem(Items.IRON_INGOT), conditionsFromItem(Items.IRON_INGOT))
                .criterion(hasItem(Items.IRON_NUGGET), conditionsFromItem(Items.IRON_NUGGET))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Custom.LIGHT_BULB_LAMP, 1)
                .input('X', Items.IRON_INGOT).input('Y', Items.IRON_NUGGET).input('Z', NSE_Items.LIGHT_BULB)
                .pattern(" X ").pattern(" Y ").pattern(" Z ")
                .criterion(hasItem(NSE_Items.LIGHT_BULB), conditionsFromItem(NSE_Items.LIGHT_BULB))
                .criterion(hasItem(Items.IRON_INGOT), conditionsFromItem(Items.IRON_INGOT))
                .criterion(hasItem(Items.IRON_NUGGET), conditionsFromItem(Items.IRON_NUGGET))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Custom.GOLDEN_LAMP, 1)
                .input('G', Items.GOLD_INGOT).input('N', Items.GOLD_NUGGET).input('Z', NSE_Items.LIGHT_BULB).input('W', Blocks.GREEN_WOOL)
                .pattern("WZW").pattern(" N ").pattern("GGG")
                .criterion(hasItem(NSE_Items.LIGHT_BULB), conditionsFromItem(NSE_Items.LIGHT_BULB))
                .criterion(hasItem(Items.GOLD_NUGGET), conditionsFromItem(Items.GOLD_NUGGET))
                .criterion(hasItem(Items.GOLD_INGOT), conditionsFromItem(Items.GOLD_INGOT))
                .criterion(hasItem(Blocks.GREEN_WOOL), conditionsFromItem(Blocks.GREEN_WOOL))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Custom.RADIO_WAR, 1)
                .input('A', NSE_Items.ANTENNA).input('N', Items.IRON_NUGGET).input('I', Items.IRON_INGOT)
                .input('R', Items.REDSTONE).input('#', Blocks.NOTE_BLOCK)
                .pattern(" A ")
                .pattern("I#I")
                .pattern("NRN")
                .criterion(hasItem(Items.IRON_NUGGET), conditionsFromItem(Items.IRON_NUGGET))
                .criterion(hasItem(Items.IRON_INGOT), conditionsFromItem(Items.IRON_INGOT))
                .criterion(hasItem(NSE_Items.ANTENNA), conditionsFromItem(NSE_Items.ANTENNA))
                .criterion(hasItem(Blocks.NOTE_BLOCK), conditionsFromItem(Blocks.NOTE_BLOCK))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Custom.TELEPHONE, 1)
                .input('N', Items.IRON_NUGGET).input('I', Items.IRON_INGOT)
                .input('D', Blocks.POLISHED_DEEPSLATE_SLAB).input('R', Items.REDSTONE).input('T', Blocks.IRON_TRAPDOOR)
                .pattern("INI")
                .pattern(" T ")
                .pattern("RDR")
                .criterion(hasItem(Items.IRON_NUGGET), conditionsFromItem(Items.IRON_NUGGET))
                .criterion(hasItem(Items.IRON_INGOT), conditionsFromItem(Items.IRON_INGOT))
                .criterion(hasItem(Blocks.POLISHED_DEEPSLATE_SLAB), conditionsFromItem(Blocks.POLISHED_DEEPSLATE_SLAB))
                .criterion(hasItem(Blocks.IRON_TRAPDOOR), conditionsFromItem(Blocks.IRON_TRAPDOOR))
                .criterion(hasItem(Items.REDSTONE), conditionsFromItem(Items.REDSTONE))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Custom.W_MACHINE, 1)
                .input('N', Items.IRON_NUGGET).input('I', Items.IRON_INGOT)
                .input('D', Blocks.POLISHED_DEEPSLATE_SLAB).input('#', Items.INK_SAC)
                .pattern(" # ")
                .pattern("IDI")
                .pattern("NNN")
                .criterion(hasItem(Items.IRON_NUGGET), conditionsFromItem(Items.IRON_NUGGET))
                .criterion(hasItem(Blocks.POLISHED_DEEPSLATE_SLAB), conditionsFromItem(Blocks.POLISHED_DEEPSLATE_SLAB))
                .criterion(hasItem(Items.IRON_INGOT), conditionsFromItem(Items.IRON_INGOT))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Custom.BIG_POST_LAMP, 1)
                .input('P', NSE_Custom.LAMP_POST_BASE).input('S', Blocks.SHROOMLIGHT)
                .pattern(" S ").pattern(" P ").pattern("   ")
                .criterion(hasItem(NSE_Custom.LAMP_POST_BASE), conditionsFromItem(NSE_Custom.LAMP_POST_BASE))
                .criterion(hasItem(Blocks.SHROOMLIGHT), conditionsFromItem(Blocks.SHROOMLIGHT))
                .offerTo(exporter,  Registries.ITEM.getId(NSE_Custom.BIG_POST_LAMP.asItem()) + "_shroomlight");

        ShapedRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Custom.BIG_POST_LAMP, 1)
                .input('P', NSE_Custom.LAMP_POST_BASE).input('S', Blocks.SEA_LANTERN)
                .pattern(" S ").pattern(" P ").pattern("   ")
                .criterion(hasItem(NSE_Custom.LAMP_POST_BASE), conditionsFromItem(NSE_Custom.LAMP_POST_BASE))
                .criterion(hasItem(Blocks.SEA_LANTERN), conditionsFromItem(Blocks.SEA_LANTERN))
                .offerTo(exporter,  Registries.ITEM.getId(NSE_Custom.BIG_POST_LAMP.asItem()) + "_sea_lantern");

        ShapedRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Custom.BIG_POST_LAMP, 1)
                .input('P', NSE_Custom.LAMP_POST_BASE).input('S', Blocks.REDSTONE_LAMP)
                .pattern(" S ").pattern(" P ").pattern("   ")
                .criterion(hasItem(NSE_Custom.LAMP_POST_BASE), conditionsFromItem(NSE_Custom.LAMP_POST_BASE))
                .criterion(hasItem(Blocks.REDSTONE_LAMP), conditionsFromItem(Blocks.REDSTONE_LAMP))
                .offerTo(exporter,  Registries.ITEM.getId(NSE_Custom.BIG_POST_LAMP.asItem()) + "_redstone_lamp");

        ShapedRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Custom.CAGED_POST_LAMP, 1)
                .input('P', NSE_Custom.LAMP_POST_BASE).input('N', Items.IRON_NUGGET).input('L', Blocks.LANTERN)
                .pattern(" N").pattern("LP")
                .criterion(hasItem(NSE_Custom.LAMP_POST_BASE), conditionsFromItem(NSE_Custom.LAMP_POST_BASE))
                .criterion(hasItem(Blocks.LANTERN), conditionsFromItem(Blocks.LANTERN))
                .criterion(hasItem(Items.IRON_NUGGET), conditionsFromItem(Items.IRON_NUGGET))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Custom.VINTAGE_POST_LAMP, 1)
                .input('P', NSE_Custom.LAMP_POST_BASE).input('N', Items.IRON_NUGGET).input('I', Items.IRON_INGOT).input('L', Blocks.LANTERN)
                .pattern("IN").pattern("LP")
                .criterion(hasItem(NSE_Custom.LAMP_POST_BASE), conditionsFromItem(NSE_Custom.LAMP_POST_BASE))
                .criterion(hasItem(Blocks.LANTERN), conditionsFromItem(Blocks.LANTERN))
                .criterion(hasItem(Items.IRON_INGOT), conditionsFromItem(Items.IRON_INGOT))
                .criterion(hasItem(Items.IRON_NUGGET), conditionsFromItem(Items.IRON_NUGGET))
                .offerTo(exporter,  Registries.ITEM.getId(NSE_Custom.VINTAGE_POST_LAMP.asItem()) + "_basic");

        ShapedRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Custom.MODERN_POST_LAMP, 1)
                .input('P', NSE_Custom.LAMP_POST_BASE).input('N', Items.IRON_NUGGET).input('I', Items.IRON_INGOT).input('L', Blocks.LANTERN)
                .pattern("INI").pattern("LP ")
                .criterion(hasItem(NSE_Custom.LAMP_POST_BASE), conditionsFromItem(NSE_Custom.LAMP_POST_BASE))
                .criterion(hasItem(Blocks.LANTERN), conditionsFromItem(Blocks.LANTERN))
                .criterion(hasItem(Items.IRON_INGOT), conditionsFromItem(Items.IRON_INGOT))
                .criterion(hasItem(Items.IRON_NUGGET), conditionsFromItem(Items.IRON_NUGGET))
                .offerTo(exporter,  Registries.ITEM.getId(NSE_Custom.MODERN_POST_LAMP.asItem()) + "_basic");

        ShapelessRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Custom.VINTAGE_POST_LAMP).input(NSE_Custom.CAGED_POST_LAMP).input(Items.IRON_INGOT).criterion(FabricRecipeProvider.hasItem(NSE_Custom.VINTAGE_POST_LAMP),
                FabricRecipeProvider.conditionsFromItem(NSE_Custom.VINTAGE_POST_LAMP)).criterion(FabricRecipeProvider.hasItem(NSE_Custom.CAGED_POST_LAMP),
                FabricRecipeProvider.conditionsFromItem(NSE_Custom.VINTAGE_POST_LAMP)).criterion(FabricRecipeProvider.hasItem(Items.IRON_INGOT),
                FabricRecipeProvider.conditionsFromItem(NSE_Custom.VINTAGE_POST_LAMP)).offerTo(exporter, Registries.ITEM.getId(NSE_Custom.VINTAGE_POST_LAMP.asItem()) + "_add_ingot");


        ShapelessRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Custom.MODERN_POST_LAMP).input(NSE_Custom.CAGED_POST_LAMP).input(Items.IRON_INGOT).input(Items.IRON_INGOT).criterion(FabricRecipeProvider.hasItem(NSE_Custom.MODERN_POST_LAMP),
                FabricRecipeProvider.conditionsFromItem(NSE_Custom.MODERN_POST_LAMP)).criterion(FabricRecipeProvider.hasItem(NSE_Custom.CAGED_POST_LAMP),
                FabricRecipeProvider.conditionsFromItem(NSE_Custom.MODERN_POST_LAMP)).criterion(FabricRecipeProvider.hasItem(Items.IRON_INGOT),
                FabricRecipeProvider.conditionsFromItem(NSE_Custom.MODERN_POST_LAMP)).offerTo(exporter, Registries.ITEM.getId(NSE_Custom.MODERN_POST_LAMP.asItem()) + "_add_ingot");

        ShapelessRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, NSE_Blocks.LEATHER_LINED_CHISELED_ACACIA_DOOR).input(NSE_Blocks.CHISELED_ACACIA_DOOR).input(Items.LEATHER, 2).criterion(FabricRecipeProvider.hasItem(NSE_Blocks.LEATHER_LINED_CHISELED_ACACIA_DOOR),
                FabricRecipeProvider.conditionsFromItem(NSE_Blocks.LEATHER_LINED_CHISELED_ACACIA_DOOR)).criterion(FabricRecipeProvider.hasItem(Items.LEATHER),
                FabricRecipeProvider.conditionsFromItem(Items.LEATHER)).offerTo(exporter, Registries.ITEM.getId(NSE_Blocks.LEATHER_LINED_CHISELED_ACACIA_DOOR.asItem()));


        ShapelessRecipeJsonBuilder.create(RecipeCategory.MISC, NSE_Items.MAKHORKA, 4).input(ItemTags.LEAVES).input(Items.GUNPOWDER).criterion(FabricRecipeProvider.hasItem(NSE_Items.MAKHORKA),
                FabricRecipeProvider.conditionsFromItem(NSE_Items.MAKHORKA)).criterion(FabricRecipeProvider.hasItem(Items.GUNPOWDER),
                FabricRecipeProvider.conditionsFromItem(Items.GUNPOWDER)).offerTo(exporter, Registries.ITEM.getId(NSE_Items.MAKHORKA.asItem()));

        ShapelessRecipeJsonBuilder.create(RecipeCategory.MISC, NSE_Items.HOMEMADE_CIGARETTE).input(NSE_Items.MAKHORKA).input(Items.PAPER).criterion(FabricRecipeProvider.hasItem(NSE_Items.HOMEMADE_CIGARETTE),
                FabricRecipeProvider.conditionsFromItem(NSE_Items.HOMEMADE_CIGARETTE)).criterion(FabricRecipeProvider.hasItem(NSE_Items.MAKHORKA),
                FabricRecipeProvider.conditionsFromItem(NSE_Items.MAKHORKA)).offerTo(exporter, Registries.ITEM.getId(NSE_Items.HOMEMADE_CIGARETTE.asItem()));

        ShapedRecipeJsonBuilder.create(RecipeCategory.MISC, NSE_Items.CIGARETTE, 1)
                .input('P', Items.PAPER).input('M', NSE_Items.MAKHORKA)
                .pattern("PMP")
                .criterion(hasItem(NSE_Items.MAKHORKA), conditionsFromItem(NSE_Items.MAKHORKA))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.MISC, NSE_Items.FRIEND_CIGARETTE, 1)
                .input('P', Items.PAPER).input('M', NSE_Items.MAKHORKA).input('B', Items.BAMBOO)
                .pattern(" M ")
                .pattern("PBP")
                .criterion(hasItem(NSE_Items.MAKHORKA), conditionsFromItem(NSE_Items.MAKHORKA))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.MISC, NSE_Items.DUCHESS_CIGARETTE, 1)
                .input('P', Items.PAPER).input('M', NSE_Items.MAKHORKA).input('K', Items.DRIED_KELP)
                .pattern(" M ")
                .pattern("PMP")
                .pattern(" K ")
                .criterion(hasItem(NSE_Items.MAKHORKA), conditionsFromItem(NSE_Items.MAKHORKA))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.MISC, NSE_Items.BELOMOR_CIGARETTE, 1)
                .input('P', Items.PAPER).input('M', NSE_Items.MAKHORKA)
                .input('K', Items.KELP).input('B', Items.BAMBOO)
                .pattern("MKM")
                .pattern("PBP")
                .criterion(hasItem(NSE_Items.MAKHORKA), conditionsFromItem(NSE_Items.MAKHORKA))
                .offerTo(exporter);


        ShapedRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Custom.SIREN, 1)
                .input('R', Items.REDSTONE).input('N', Blocks.NOTE_BLOCK).input('I', Items.IRON_NUGGET)
                .pattern("   ").pattern("IRI").pattern("NIN")
                .criterion(hasItem(Items.REDSTONE), conditionsFromItem(Items.REDSTONE))
                .criterion(hasItem(Items.IRON_NUGGET), conditionsFromItem(Items.IRON_NUGGET))
                .criterion(hasItem(Blocks.NOTE_BLOCK), conditionsFromItem(Blocks.NOTE_BLOCK))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Blocks.HANDRAIL, 2)
                .input('S', ItemTags.WOODEN_SLABS).input('B', Blocks.IRON_BARS)
                .pattern(" S ").pattern(" B ").pattern("   ")
                .criterion(hasItem(Blocks.IRON_BARS), conditionsFromItem(Blocks.IRON_BARS))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, NSE_Blocks.NII_FLOOR, 4)
                .input('#', Items.DIORITE).input('G', Items.GRANITE).input('C', Items.COBBLESTONE)
                .pattern("#G").pattern("GC")
                .criterion(hasItem(Items.GRANITE), conditionsFromItem(Items.GRANITE))
                .criterion(hasItem(Items.DIORITE), conditionsFromItem(Items.DIORITE))
                .criterion(hasItem(Items.COBBLESTONE), conditionsFromItem(Items.COBBLESTONE))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.COMBAT, NSE_Blocks.BARBED_WIRE, 2)
                .input('I', Items.IRON_INGOT).input('N', Items.IRON_NUGGET)
                .pattern("INI").pattern("N N").pattern("INI")
                .criterion(hasItem(Items.IRON_INGOT), conditionsFromItem(Items.IRON_INGOT))
                .criterion(hasItem(Items.IRON_NUGGET), conditionsFromItem(Items.IRON_NUGGET))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.COMBAT, NSE_Custom.AP_LANDMINE, 6)
                .input('P', Blocks.STONE_PRESSURE_PLATE).input('N', Items.IRON_NUGGET).input('T', Blocks.TNT)
                .pattern("NPN").pattern("TTT").pattern("NNN")
                .criterion(hasItem(Items.STONE_PRESSURE_PLATE), conditionsFromItem(Items.STONE_PRESSURE_PLATE))
                .criterion(hasItem(Items.IRON_NUGGET), conditionsFromItem(Items.IRON_NUGGET))
                .criterion(hasItem(Blocks.TNT), conditionsFromItem(Blocks.TNT))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Custom.SWITCH, 1)
                .pattern(" B ")
                .pattern(" R ")
                .pattern(" S ")
                .input('S', Blocks.OAK_SLAB)
                .input('R', Items.REDSTONE)
                .input('B', Blocks.STONE_BUTTON)
                .criterion(RecipeProvider.hasItem(Blocks.OAK_SLAB),
                        RecipeProvider.conditionsFromItem(Blocks.OAK_SLAB))
                .criterion(RecipeProvider.hasItem(Items.REDSTONE),
                        RecipeProvider.conditionsFromItem(Items.REDSTONE))
                .criterion(RecipeProvider.hasItem(Blocks.STONE_BUTTON),
                        RecipeProvider.conditionsFromItem(Blocks.STONE_BUTTON))
                .offerTo(exporter);

        ShapedRecipeJsonBuilder.create(RecipeCategory.DECORATIONS, NSE_Custom.DARK_SWITCH, 1)
                .pattern(" B ")
                .pattern(" R ")
                .pattern(" S ")
                .input('S', Blocks.DARK_OAK_SLAB)
                .input('R', Items.REDSTONE)
                .input('B', Blocks.STONE_BUTTON)
                .criterion(RecipeProvider.hasItem(Blocks.DARK_OAK_SLAB),
                        RecipeProvider.conditionsFromItem(Blocks.DARK_OAK_SLAB))
                .criterion(RecipeProvider.hasItem(Items.REDSTONE),
                        RecipeProvider.conditionsFromItem(Items.REDSTONE))
                .criterion(RecipeProvider.hasItem(Blocks.STONE_BUTTON),
                        RecipeProvider.conditionsFromItem(Blocks.STONE_BUTTON))
                .offerTo(exporter);

        var path_blue_bars = convertBetween(NSE_Blocks.BLUE_IRON_BARS, Blocks.IRON_BARS);
        ShapelessRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, (NSE_Blocks.BLUE_IRON_BARS)).input(Blocks.IRON_BARS).input(Items.BLUE_DYE).criterion(FabricRecipeProvider.hasItem(NSE_Blocks.BLUE_IRON_BARS),
                FabricRecipeProvider.conditionsFromItem(NSE_Blocks.BLUE_IRON_BARS)).criterion(FabricRecipeProvider.hasItem(Blocks.IRON_BARS),
                FabricRecipeProvider.conditionsFromItem(NSE_Blocks.BLUE_IRON_BARS)).criterion(FabricRecipeProvider.hasItem(Items.BLUE_DYE),
                FabricRecipeProvider.conditionsFromItem(NSE_Blocks.BLUE_IRON_BARS)).offerTo(exporter, path_blue_bars + "_blue");
        ShapelessRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, (NSE_Blocks.BLUE_IRON_BARS)).input(Blocks.IRON_BARS).input(Items.LIGHT_BLUE_DYE).criterion(FabricRecipeProvider.hasItem(NSE_Blocks.BLUE_IRON_BARS),
                FabricRecipeProvider.conditionsFromItem(NSE_Blocks.BLUE_IRON_BARS)).criterion(FabricRecipeProvider.hasItem(Blocks.IRON_BARS),
                FabricRecipeProvider.conditionsFromItem(NSE_Blocks.BLUE_IRON_BARS)).criterion(FabricRecipeProvider.hasItem(Items.LIGHT_BLUE_DYE),
                FabricRecipeProvider.conditionsFromItem(NSE_Blocks.BLUE_IRON_BARS)).offerTo(exporter, path_blue_bars + "_light_blue");
        ShapelessRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, (NSE_Blocks.BLUE_IRON_BARS)).input(Blocks.IRON_BARS).input(Items.CYAN_DYE).criterion(FabricRecipeProvider.hasItem(NSE_Blocks.BLUE_IRON_BARS),
                FabricRecipeProvider.conditionsFromItem(NSE_Blocks.BLUE_IRON_BARS)).criterion(FabricRecipeProvider.hasItem(Blocks.IRON_BARS),
                FabricRecipeProvider.conditionsFromItem(NSE_Blocks.BLUE_IRON_BARS)).criterion(FabricRecipeProvider.hasItem(Items.CYAN_DYE),
                FabricRecipeProvider.conditionsFromItem(NSE_Blocks.BLUE_IRON_BARS)).offerTo(exporter, path_blue_bars + "_cyan");

        centerRecipe(exporter, NSE_Blocks.BIG_GREEN_TILES, NSE_Blocks.BIG_DIORITE_TILES, Items.GREEN_DYE);
        centerRecipe(exporter, NSE_Blocks.GREEN_BRICKS, Items.BRICKS, Items.GREEN_DYE);
        centerRecipe(exporter, NSE_Blocks.SMALL_YELLOW_TILES, NSE_Blocks.SMALL_WHITE_TILES, Items.YELLOW_DYE);
        centerRecipe(exporter, NSE_Blocks.RED_BRICKS, Items.BRICKS, Items.RED_DYE);
        centerRecipe(exporter, NSE_Blocks.DARK_BRICK_TILES, NSE_Blocks.BRICK_TILES, Items.BROWN_DYE);

        ShapelessRecipeJsonBuilder.create(RecipeCategory.BUILDING_BLOCKS, NSE_Blocks.GREEN_BRICKS_MIXED, 2)
                .input(Items.BRICKS).input(NSE_Blocks.GREEN_BRICKS)
                .criterion(hasItem(NSE_Blocks.GREEN_BRICKS), conditionsFromItem(NSE_Blocks.GREEN_BRICKS))
                .criterion(hasItem(Items.BRICKS), conditionsFromItem(Items.BRICKS))
                .offerTo(exporter);

        // Generated stairs and slabs recipes
        offerStairsAndSlab(exporter, NSE_Blocks.SAND_TILES_STAIRS, NSE_Blocks.SAND_TILES_SLAB, NSE_Blocks.SAND_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_SAND_TILES_STAIRS, NSE_Blocks.CRACKED_SAND_TILES_SLAB, NSE_Blocks.CRACKED_SAND_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.MOSSY_SAND_TILES_STAIRS, NSE_Blocks.MOSSY_SAND_TILES_SLAB, NSE_Blocks.MOSSY_SAND_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.HERRINGBONE_SAND_TILES_STAIRS, NSE_Blocks.HERRINGBONE_SAND_TILES_SLAB, NSE_Blocks.HERRINGBONE_SAND_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.CROSS_SAND_TILES_STAIRS, NSE_Blocks.CROSS_SAND_TILES_SLAB, NSE_Blocks.CROSS_SAND_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.SAND_BRICKS_STAIRS, NSE_Blocks.SAND_BRICKS_SLAB, NSE_Blocks.SAND_BRICKS);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_SAND_BRICKS_STAIRS, NSE_Blocks.CRACKED_SAND_BRICKS_SLAB, NSE_Blocks.CRACKED_SAND_BRICKS);
        offerStairsAndSlab(exporter, NSE_Blocks.MOSSY_SAND_BRICKS_STAIRS, NSE_Blocks.MOSSY_SAND_BRICKS_SLAB, NSE_Blocks.MOSSY_SAND_BRICKS);
        offerStairsAndSlab(exporter, NSE_Blocks.BIG_SAND_BRICKS_STAIRS, NSE_Blocks.BIG_SAND_BRICKS_SLAB, NSE_Blocks.BIG_SAND_BRICKS);
        offerStairsAndSlab(exporter, NSE_Blocks.BRICK_TILES_STAIRS, NSE_Blocks.BRICK_TILES_SLAB, NSE_Blocks.BRICK_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_BRICK_TILES_STAIRS, NSE_Blocks.CRACKED_BRICK_TILES_SLAB, NSE_Blocks.CRACKED_BRICK_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.MOSSY_BRICK_TILES_STAIRS, NSE_Blocks.MOSSY_BRICK_TILES_SLAB, NSE_Blocks.MOSSY_BRICK_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.DIRTY_BRICK_TILES_STAIRS, NSE_Blocks.DIRTY_BRICK_TILES_SLAB, NSE_Blocks.DIRTY_BRICK_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.DARK_BRICK_TILES_STAIRS, NSE_Blocks.DARK_BRICK_TILES_SLAB, NSE_Blocks.DARK_BRICK_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_DARK_BRICK_TILES_STAIRS, NSE_Blocks.CRACKED_DARK_BRICK_TILES_SLAB, NSE_Blocks.CRACKED_DARK_BRICK_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.MOSSY_DARK_BRICK_TILES_STAIRS, NSE_Blocks.MOSSY_DARK_BRICK_TILES_SLAB, NSE_Blocks.MOSSY_DARK_BRICK_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.DIRTY_DARK_BRICK_TILES_STAIRS, NSE_Blocks.DIRTY_DARK_BRICK_TILES_SLAB, NSE_Blocks.DIRTY_DARK_BRICK_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.RED_BRICKS_STAIRS, NSE_Blocks.RED_BRICKS_SLAB, NSE_Blocks.RED_BRICKS);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_RED_BRICKS_STAIRS, NSE_Blocks.CRACKED_RED_BRICKS_SLAB, NSE_Blocks.CRACKED_RED_BRICKS);
        offerStairsAndSlab(exporter, NSE_Blocks.MOSSY_RED_BRICKS_STAIRS, NSE_Blocks.MOSSY_RED_BRICKS_SLAB, NSE_Blocks.MOSSY_RED_BRICKS);
        offerStairsAndSlab(exporter, NSE_Blocks.TEAL_TILES_STAIRS, NSE_Blocks.TEAL_TILES_SLAB, NSE_Blocks.TEAL_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_TEAL_TILES_STAIRS, NSE_Blocks.CRACKED_TEAL_TILES_SLAB, NSE_Blocks.CRACKED_TEAL_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.MOSSY_TEAL_TILES_STAIRS, NSE_Blocks.MOSSY_TEAL_TILES_SLAB, NSE_Blocks.MOSSY_TEAL_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.VARIATED_TEAL_TILES_STAIRS, NSE_Blocks.VARIATED_TEAL_TILES_SLAB, NSE_Blocks.VARIATED_TEAL_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.GREEN_BRICKS_STAIRS, NSE_Blocks.GREEN_BRICKS_SLAB, NSE_Blocks.GREEN_BRICKS);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_GREEN_BRICKS_STAIRS, NSE_Blocks.CRACKED_GREEN_BRICKS_SLAB, NSE_Blocks.CRACKED_GREEN_BRICKS);
        offerStairsAndSlab(exporter, NSE_Blocks.MOSSY_GREEN_BRICKS_STAIRS, NSE_Blocks.MOSSY_GREEN_BRICKS_SLAB, NSE_Blocks.MOSSY_GREEN_BRICKS);
        offerStairsAndSlab(exporter, NSE_Blocks.GREEN_BRICKS_MIXED_STAIRS, NSE_Blocks.GREEN_BRICKS_MIXED_SLAB, NSE_Blocks.GREEN_BRICKS_MIXED);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_GREEN_BRICKS_MIXED_STAIRS, NSE_Blocks.CRACKED_GREEN_BRICKS_MIXED_SLAB, NSE_Blocks.CRACKED_GREEN_BRICKS_MIXED);
        offerStairsAndSlab(exporter, NSE_Blocks.MOSSY_GREEN_BRICKS_MIXED_STAIRS, NSE_Blocks.MOSSY_GREEN_BRICKS_MIXED_SLAB, NSE_Blocks.MOSSY_GREEN_BRICKS_MIXED);
        offerStairsAndSlab(exporter, NSE_Blocks.AQUAMARINE_TILES_STAIRS, NSE_Blocks.AQUAMARINE_TILES_SLAB, NSE_Blocks.AQUAMARINE_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_AQUAMARINE_TILES_STAIRS, NSE_Blocks.CRACKED_AQUAMARINE_TILES_SLAB, NSE_Blocks.CRACKED_AQUAMARINE_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.MOSSY_AQUAMARINE_TILES_STAIRS, NSE_Blocks.MOSSY_AQUAMARINE_TILES_SLAB, NSE_Blocks.MOSSY_AQUAMARINE_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.DIRTY_AQUAMARINE_TILES_STAIRS, NSE_Blocks.DIRTY_AQUAMARINE_TILES_SLAB, NSE_Blocks.DIRTY_AQUAMARINE_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.DIORITE_BRICKS_STAIRS, NSE_Blocks.DIORITE_BRICKS_SLAB, NSE_Blocks.DIORITE_BRICKS);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_DIORITE_BRICKS_STAIRS, NSE_Blocks.CRACKED_DIORITE_BRICKS_SLAB, NSE_Blocks.CRACKED_DIORITE_BRICKS);
        offerStairsAndSlab(exporter, NSE_Blocks.MOSSY_DIORITE_BRICKS_STAIRS, NSE_Blocks.MOSSY_DIORITE_BRICKS_SLAB, NSE_Blocks.MOSSY_DIORITE_BRICKS);
        offerStairsAndSlab(exporter, NSE_Blocks.CALCITE_TILES_STAIRS, NSE_Blocks.CALCITE_TILES_SLAB, NSE_Blocks.CALCITE_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_CALCITE_TILES_STAIRS, NSE_Blocks.CRACKED_CALCITE_TILES_SLAB, NSE_Blocks.CRACKED_CALCITE_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.MOSSY_CALCITE_TILES_STAIRS, NSE_Blocks.MOSSY_CALCITE_TILES_SLAB, NSE_Blocks.MOSSY_CALCITE_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.DRIPSTONE_TILES_STAIRS, NSE_Blocks.DRIPSTONE_TILES_SLAB, NSE_Blocks.DRIPSTONE_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_DRIPSTONE_TILES_STAIRS, NSE_Blocks.CRACKED_DRIPSTONE_TILES_SLAB, NSE_Blocks.CRACKED_DRIPSTONE_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.MOSSY_DRIPSTONE_TILES_STAIRS, NSE_Blocks.MOSSY_DRIPSTONE_TILES_SLAB, NSE_Blocks.MOSSY_DRIPSTONE_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.DEEPSLATE_TILES_STAIRS, NSE_Blocks.DEEPSLATE_TILES_SLAB, NSE_Blocks.DEEPSLATE_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_DEEPSLATE_TILES_STAIRS, NSE_Blocks.CRACKED_DEEPSLATE_TILES_SLAB, NSE_Blocks.CRACKED_DEEPSLATE_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.MOSSY_DEEPSLATE_TILES_STAIRS, NSE_Blocks.MOSSY_DEEPSLATE_TILES_SLAB, NSE_Blocks.MOSSY_DEEPSLATE_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.LIGHT_BLUE_TILES_STAIRS, NSE_Blocks.LIGHT_BLUE_TILES_SLAB, NSE_Blocks.LIGHT_BLUE_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_LIGHT_BLUE_TILES_STAIRS, NSE_Blocks.CRACKED_LIGHT_BLUE_TILES_SLAB, NSE_Blocks.CRACKED_LIGHT_BLUE_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.MOSSY_LIGHT_BLUE_TILES_STAIRS, NSE_Blocks.MOSSY_LIGHT_BLUE_TILES_SLAB, NSE_Blocks.MOSSY_LIGHT_BLUE_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.VARIATED_LIGHT_BLUE_TILES_STAIRS, NSE_Blocks.VARIATED_LIGHT_BLUE_TILES_SLAB, NSE_Blocks.VARIATED_LIGHT_BLUE_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.LIGHT_BLUE_BRICKS_STAIRS, NSE_Blocks.LIGHT_BLUE_BRICKS_SLAB, NSE_Blocks.LIGHT_BLUE_BRICKS);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_LIGHT_BLUE_BRICKS_STAIRS, NSE_Blocks.CRACKED_LIGHT_BLUE_BRICKS_SLAB, NSE_Blocks.CRACKED_LIGHT_BLUE_BRICKS);
        offerStairsAndSlab(exporter, NSE_Blocks.MOSSY_LIGHT_BLUE_BRICKS_STAIRS, NSE_Blocks.MOSSY_LIGHT_BLUE_BRICKS_SLAB, NSE_Blocks.MOSSY_LIGHT_BLUE_BRICKS);
        offerStairsAndSlab(exporter, NSE_Blocks.GREEN_WHITE_TILES_STAIRS, NSE_Blocks.GREEN_WHITE_TILES_SLAB, NSE_Blocks.GREEN_WHITE_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_GREEN_WHITE_TILES_STAIRS, NSE_Blocks.CRACKED_GREEN_WHITE_TILES_SLAB, NSE_Blocks.CRACKED_GREEN_WHITE_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.MOSSY_GREEN_WHITE_TILES_STAIRS, NSE_Blocks.MOSSY_GREEN_WHITE_TILES_SLAB, NSE_Blocks.MOSSY_GREEN_WHITE_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.TUFF_TILES_STAIRS, NSE_Blocks.TUFF_TILES_SLAB, NSE_Blocks.TUFF_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_TUFF_TILES_STAIRS, NSE_Blocks.CRACKED_TUFF_TILES_SLAB, NSE_Blocks.CRACKED_TUFF_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.MOSSY_TUFF_TILES_STAIRS, NSE_Blocks.MOSSY_TUFF_TILES_SLAB, NSE_Blocks.MOSSY_TUFF_TILES);
        offerStairsAndSlab(exporter, NSE_Blocks.TUFF_BRICKS_STAIRS, NSE_Blocks.TUFF_BRICKS_SLAB, NSE_Blocks.TUFF_BRICKS);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_TUFF_BRICKS_STAIRS, NSE_Blocks.CRACKED_TUFF_BRICKS_SLAB, NSE_Blocks.CRACKED_TUFF_BRICKS);
        offerStairsAndSlab(exporter, NSE_Blocks.MOSSY_TUFF_BRICKS_STAIRS, NSE_Blocks.MOSSY_TUFF_BRICKS_SLAB, NSE_Blocks.MOSSY_TUFF_BRICKS);
        offerStairsAndSlab(exporter, NSE_Blocks.CONCRETE_STAIRS, NSE_Blocks.CONCRETE_SLAB, NSE_Blocks.CONCRETE);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_CONCRETE_STAIRS, NSE_Blocks.CRACKED_CONCRETE_SLAB, NSE_Blocks.CRACKED_CONCRETE);
        offerStairsAndSlab(exporter, NSE_Blocks.WHITE_CONCRETE_STAIRS, NSE_Blocks.WHITE_CONCRETE_SLAB, NSE_Blocks.WHITE_CONCRETE);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_WHITE_CONCRETE_STAIRS, NSE_Blocks.CRACKED_WHITE_CONCRETE_SLAB, NSE_Blocks.CRACKED_WHITE_CONCRETE);
        offerStairsAndSlab(exporter, NSE_Blocks.BEIGE_CONCRETE_STAIRS, NSE_Blocks.BEIGE_CONCRETE_SLAB, NSE_Blocks.BEIGE_CONCRETE);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_BEIGE_CONCRETE_STAIRS, NSE_Blocks.CRACKED_BEIGE_CONCRETE_SLAB, NSE_Blocks.CRACKED_BEIGE_CONCRETE);
        offerStairsAndSlab(exporter, NSE_Blocks.BLUE_CONCRETE_STAIRS, NSE_Blocks.BLUE_CONCRETE_SLAB, NSE_Blocks.BLUE_CONCRETE);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_BLUE_CONCRETE_STAIRS, NSE_Blocks.CRACKED_BLUE_CONCRETE_SLAB, NSE_Blocks.CRACKED_BLUE_CONCRETE);
        offerStairsAndSlab(exporter, NSE_Blocks.RED_CONCRETE_STAIRS, NSE_Blocks.RED_CONCRETE_SLAB, NSE_Blocks.RED_CONCRETE);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_RED_CONCRETE_STAIRS, NSE_Blocks.CRACKED_RED_CONCRETE_SLAB, NSE_Blocks.CRACKED_RED_CONCRETE);
        offerStairsAndSlab(exporter, NSE_Blocks.YELLOW_CONCRETE_STAIRS, NSE_Blocks.YELLOW_CONCRETE_SLAB, NSE_Blocks.YELLOW_CONCRETE);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_YELLOW_CONCRETE_STAIRS, NSE_Blocks.CRACKED_YELLOW_CONCRETE_SLAB, NSE_Blocks.CRACKED_YELLOW_CONCRETE);
        offerStairsAndSlab(exporter, NSE_Blocks.GREEN_CONCRETE_STAIRS, NSE_Blocks.GREEN_CONCRETE_SLAB, NSE_Blocks.GREEN_CONCRETE);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_GREEN_CONCRETE_STAIRS, NSE_Blocks.CRACKED_GREEN_CONCRETE_SLAB, NSE_Blocks.CRACKED_GREEN_CONCRETE);
        offerStairsAndSlab(exporter, NSE_Blocks.DARK_GREEN_CONCRETE_STAIRS, NSE_Blocks.DARK_GREEN_CONCRETE_SLAB, NSE_Blocks.DARK_GREEN_CONCRETE);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_DARK_GREEN_CONCRETE_STAIRS, NSE_Blocks.CRACKED_DARK_GREEN_CONCRETE_SLAB, NSE_Blocks.CRACKED_DARK_GREEN_CONCRETE);
        offerStairsAndSlab(exporter, NSE_Blocks.ORANGE_CONCRETE_STAIRS, NSE_Blocks.ORANGE_CONCRETE_SLAB, NSE_Blocks.ORANGE_CONCRETE);
        offerStairsAndSlab(exporter, NSE_Blocks.CRACKED_ORANGE_CONCRETE_STAIRS, NSE_Blocks.CRACKED_ORANGE_CONCRETE_SLAB, NSE_Blocks.CRACKED_ORANGE_CONCRETE);
        offerStairsAndSlab(exporter, NSE_Blocks.NII_FLOOR_STAIRS, NSE_Blocks.NII_FLOOR_SLAB, NSE_Blocks.NII_FLOOR);
        offerStairsAndSlab(exporter, NSE_Blocks.HERRINGBONE_ACACIA_PLANKS_STAIRS, NSE_Blocks.HERRINGBONE_ACACIA_PLANKS_SLAB, NSE_Blocks.HERRINGBONE_ACACIA_PLANKS);
        offerStairsAndSlab(exporter, NSE_Blocks.CROSS_ACACIA_PLANKS_STAIRS, NSE_Blocks.CROSS_ACACIA_PLANKS_SLAB, NSE_Blocks.CROSS_ACACIA_PLANKS);
        offerStairsAndSlab(exporter, NSE_Blocks.HERRINGBONE_OAK_PLANKS_STAIRS, NSE_Blocks.HERRINGBONE_OAK_PLANKS_SLAB, NSE_Blocks.HERRINGBONE_OAK_PLANKS);
        offerStairsAndSlab(exporter, NSE_Blocks.CROSS_OAK_PLANKS_STAIRS, NSE_Blocks.CROSS_OAK_PLANKS_SLAB, NSE_Blocks.CROSS_OAK_PLANKS);
        offerStairsAndSlab(exporter, NSE_Blocks.HERRINGBONE_BIRCH_PLANKS_STAIRS, NSE_Blocks.HERRINGBONE_BIRCH_PLANKS_SLAB, NSE_Blocks.HERRINGBONE_BIRCH_PLANKS);
        offerStairsAndSlab(exporter, NSE_Blocks.CROSS_BIRCH_PLANKS_STAIRS, NSE_Blocks.CROSS_BIRCH_PLANKS_SLAB, NSE_Blocks.CROSS_BIRCH_PLANKS);
        offerStairsAndSlab(exporter, NSE_Blocks.HERRINGBONE_CRIMSON_PLANKS_STAIRS, NSE_Blocks.HERRINGBONE_CRIMSON_PLANKS_SLAB, NSE_Blocks.HERRINGBONE_CRIMSON_PLANKS);
        offerStairsAndSlab(exporter, NSE_Blocks.CROSS_CRIMSON_PLANKS_STAIRS, NSE_Blocks.CROSS_CRIMSON_PLANKS_SLAB, NSE_Blocks.CROSS_CRIMSON_PLANKS);
        offerStairsAndSlab(exporter, NSE_Blocks.HERRINGBONE_WARPED_PLANKS_STAIRS, NSE_Blocks.HERRINGBONE_WARPED_PLANKS_SLAB, NSE_Blocks.HERRINGBONE_WARPED_PLANKS);
        offerStairsAndSlab(exporter, NSE_Blocks.CROSS_WARPED_PLANKS_STAIRS, NSE_Blocks.CROSS_WARPED_PLANKS_SLAB, NSE_Blocks.CROSS_WARPED_PLANKS);
        offerStairsAndSlab(exporter, NSE_Blocks.HERRINGBONE_DARK_OAK_PLANKS_STAIRS, NSE_Blocks.HERRINGBONE_DARK_OAK_PLANKS_SLAB, NSE_Blocks.HERRINGBONE_DARK_OAK_PLANKS);
        offerStairsAndSlab(exporter, NSE_Blocks.CROSS_DARK_OAK_PLANKS_STAIRS, NSE_Blocks.CROSS_DARK_OAK_PLANKS_SLAB, NSE_Blocks.CROSS_DARK_OAK_PLANKS);
        offerStairsAndSlab(exporter, NSE_Blocks.HERRINGBONE_JUNGLE_PLANKS_STAIRS, NSE_Blocks.HERRINGBONE_JUNGLE_PLANKS_SLAB, NSE_Blocks.HERRINGBONE_JUNGLE_PLANKS);
        offerStairsAndSlab(exporter, NSE_Blocks.CROSS_JUNGLE_PLANKS_STAIRS, NSE_Blocks.CROSS_JUNGLE_PLANKS_SLAB, NSE_Blocks.CROSS_JUNGLE_PLANKS);
        offerStairsAndSlab(exporter, NSE_Blocks.HERRINGBONE_MANGROVE_PLANKS_STAIRS, NSE_Blocks.HERRINGBONE_MANGROVE_PLANKS_SLAB, NSE_Blocks.HERRINGBONE_MANGROVE_PLANKS);
        offerStairsAndSlab(exporter, NSE_Blocks.CROSS_MANGROVE_PLANKS_STAIRS, NSE_Blocks.CROSS_MANGROVE_PLANKS_SLAB, NSE_Blocks.CROSS_MANGROVE_PLANKS);
        offerStairsAndSlab(exporter, NSE_Blocks.HERRINGBONE_SPRUCE_PLANKS_STAIRS, NSE_Blocks.HERRINGBONE_SPRUCE_PLANKS_SLAB, NSE_Blocks.HERRINGBONE_SPRUCE_PLANKS);
        offerStairsAndSlab(exporter, NSE_Blocks.CROSS_SPRUCE_PLANKS_STAIRS, NSE_Blocks.CROSS_SPRUCE_PLANKS_SLAB, NSE_Blocks.CROSS_SPRUCE_PLANKS);
        offerStairsAndSlab(exporter, NSE_Blocks.HERRINGBONE_DARK_OAK_PARQUET_STAIRS, NSE_Blocks.HERRINGBONE_DARK_OAK_PARQUET_SLAB, NSE_Blocks.HERRINGBONE_DARK_OAK_PARQUET);
        offerStairsAndSlab(exporter, NSE_Blocks.STRAIGHT_OAK_PARQUET_STAIRS, NSE_Blocks.STRAIGHT_OAK_PARQUET_SLAB, NSE_Blocks.STRAIGHT_OAK_PARQUET);
        offerStairsAndSlab(exporter, NSE_Blocks.SEPARATED_OAK_PARQUET_STAIRS, NSE_Blocks.SEPARATED_OAK_PARQUET_SLAB, NSE_Blocks.SEPARATED_OAK_PARQUET);
        offerStairsAndSlab(exporter, NSE_Blocks.GREEN_LINOLEUM_STAIRS, NSE_Blocks.GREEN_LINOLEUM_SLAB, NSE_Blocks.GREEN_LINOLEUM);
        offerStairsAndSlab(exporter, NSE_Blocks.BLUE_LINOLEUM_STAIRS, NSE_Blocks.BLUE_LINOLEUM_SLAB, NSE_Blocks.BLUE_LINOLEUM);
        offerStairsAndSlab(exporter, NSE_Blocks.RED_LINOLEUM_STAIRS, NSE_Blocks.RED_LINOLEUM_SLAB, NSE_Blocks.RED_LINOLEUM);
        offerStairsAndSlab(exporter, NSE_Blocks.GRAY_LINOLEUM_STAIRS, NSE_Blocks.GRAY_LINOLEUM_SLAB, NSE_Blocks.GRAY_LINOLEUM);
        offerStairsAndSlab(exporter, NSE_Blocks.ORANGE_LINOLEUM_STAIRS, NSE_Blocks.ORANGE_LINOLEUM_SLAB, NSE_Blocks.ORANGE_LINOLEUM);
        offerStairsAndSlab(exporter, NSE_Blocks.BROWN_LINOLEUM_STAIRS, NSE_Blocks.BROWN_LINOLEUM_SLAB, NSE_Blocks.BROWN_LINOLEUM);
        offerStairsAndSlab(exporter, NSE_Blocks.CYAN_LINOLEUM_STAIRS, NSE_Blocks.CYAN_LINOLEUM_SLAB, NSE_Blocks.CYAN_LINOLEUM);
        offerStairsAndSlab(exporter, NSE_Blocks.METAL_PLATING_STAIRS, NSE_Blocks.METAL_PLATING_SLAB, NSE_Blocks.METAL_PLATING);
        offerStairsAndSlab(exporter, NSE_Blocks.HERRINGBONE_CHERRY_PLANKS_STAIRS, NSE_Blocks.HERRINGBONE_CHERRY_PLANKS_SLAB, NSE_Blocks.HERRINGBONE_CHERRY_PLANKS);
        offerStairsAndSlab(exporter, NSE_Blocks.CROSS_CHERRY_PLANKS_STAIRS, NSE_Blocks.CROSS_CHERRY_PLANKS_SLAB, NSE_Blocks.CROSS_CHERRY_PLANKS);
        offerStairsAndSlab(exporter, NSE_Blocks.HERRINGBONE_BAMBOO_PLANKS_STAIRS, NSE_Blocks.HERRINGBONE_BAMBOO_PLANKS_SLAB, NSE_Blocks.HERRINGBONE_BAMBOO_PLANKS);

        offerStonecuttingRecipe(exporter, RecipeCategory.BUILDING_BLOCKS, NSE_Blocks.WHITE_BOUNDARY_MARKER, Blocks.DIORITE_WALL, 1);
        offerStonecuttingRecipe(exporter, RecipeCategory.BUILDING_BLOCKS, NSE_Blocks.WHITE_BOUNDARY_MARKER, Blocks.BONE_BLOCK, 2);

        offerStonecuttingRecipe(exporter, RecipeCategory.BUILDING_BLOCKS, NSE_Custom.LAMP_POST_BASE, Blocks.IRON_BARS, 1);
        offerStonecuttingRecipe(exporter, RecipeCategory.BUILDING_BLOCKS, NSE_Custom.LAMP_POST_BASE, NSE_Blocks.VINTAGE_IRON_BARS, 1);

        offerDyeableRecipes(exporter,
                List.of(Items.GREEN_DYE, Items.BLACK_DYE, Items.YELLOW_DYE, Items.BLUE_DYE, Items.LIGHT_BLUE_DYE, Items.RED_DYE, Items.WHITE_DYE, Items.LIGHT_GRAY_DYE, Items.GRAY_DYE, Items.CYAN_DYE, Items.ORANGE_DYE, Items.PURPLE_DYE, Items.MAGENTA_DYE, Items.PINK_DYE, Items.LIME_DYE, Items.BROWN_DYE),
                List.of(NSE_Blocks.GREEN_BOUNDARY_MARKER.asItem(), NSE_Blocks.BLACK_BOUNDARY_MARKER.asItem(), NSE_Blocks.YELLOW_BOUNDARY_MARKER.asItem(), NSE_Blocks.BLUE_BOUNDARY_MARKER.asItem(),
                        NSE_Blocks.LIGHT_BLUE_BOUNDARY_MARKER.asItem(), NSE_Blocks.RED_BOUNDARY_MARKER.asItem(), NSE_Blocks.WHITE_BOUNDARY_MARKER.asItem(), NSE_Blocks.LIGHT_GRAY_BOUNDARY_MARKER.asItem(), NSE_Blocks.GRAY_BOUNDARY_MARKER.asItem(),
                        NSE_Blocks.CYAN_BOUNDARY_MARKER.asItem(), NSE_Blocks.ORANGE_BOUNDARY_MARKER.asItem(), NSE_Blocks.PURPLE_BOUNDARY_MARKER.asItem(), NSE_Blocks.MAGENTA_BOUNDARY_MARKER.asItem(), NSE_Blocks.PINK_BOUNDARY_MARKER.asItem(), NSE_Blocks.LIME_BOUNDARY_MARKER.asItem(), NSE_Blocks.BROWN_BOUNDARY_MARKER.asItem()),
                "boundary_markers");

        offerDyeableRecipes(exporter,
                List.of(Items.GREEN_DYE, Items.BLACK_DYE, Items.YELLOW_DYE, Items.BLUE_DYE, Items.LIGHT_BLUE_DYE, Items.RED_DYE, Items.GRAY_DYE, Items.CYAN_DYE, Items.ORANGE_DYE, Items.PURPLE_DYE, Items.MAGENTA_DYE, Items.LIME_DYE),
                List.of(NSE_Blocks.GREEN_WARNING.asItem(), NSE_Blocks.INDUSTRIAL_WARNING.asItem(), NSE_Blocks.YELLOW_WARNING.asItem(), NSE_Blocks.BLUE_WARNING.asItem(),
                        NSE_Blocks.LIGHT_BLUE_WARNING.asItem(), NSE_Blocks.RED_WARNING.asItem(), NSE_Blocks.GRAY_WARNING.asItem(),
                        NSE_Blocks.CYAN_WARNING.asItem(), NSE_Blocks.ORANGE_WARNING.asItem(), NSE_Blocks.PURPLE_WARNING.asItem(), NSE_Blocks.MAGENTA_WARNING.asItem(), NSE_Blocks.LIME_WARNING.asItem()),
                "warning_stripes");

        offerDyeableRecipes(exporter,
                List.of(Items.LIME_DYE, Items.YELLOW_DYE, Items.BLUE_DYE, Items.RED_DYE, Items.WHITE_DYE, Items.LIGHT_GRAY_DYE,
                        Items.GREEN_DYE, Items.ORANGE_DYE),
                List.of(NSE_Blocks.GREEN_CONCRETE.asItem(), NSE_Blocks.YELLOW_CONCRETE.asItem(), NSE_Blocks.BLUE_CONCRETE.asItem(),
                        NSE_Blocks.RED_CONCRETE.asItem(), NSE_Blocks.WHITE_CONCRETE.asItem(), NSE_Blocks.BEIGE_CONCRETE.asItem(),
                        NSE_Blocks.DARK_GREEN_CONCRETE.asItem(), NSE_Blocks.ORANGE_CONCRETE.asItem()),
                "concrete");
        offerDyeableRecipes(exporter,
                List.of(Items.LIME_DYE, Items.YELLOW_DYE, Items.BLUE_DYE, Items.RED_DYE, Items.WHITE_DYE, Items.LIGHT_GRAY_DYE,
                        Items.GREEN_DYE, Items.ORANGE_DYE),
                List.of(NSE_Blocks.CRACKED_GREEN_CONCRETE.asItem(), NSE_Blocks.CRACKED_YELLOW_CONCRETE.asItem(), NSE_Blocks.CRACKED_BLUE_CONCRETE.asItem(),
                        NSE_Blocks.CRACKED_RED_CONCRETE.asItem(), NSE_Blocks.CRACKED_WHITE_CONCRETE.asItem(), NSE_Blocks.CRACKED_BEIGE_CONCRETE.asItem(),
                        NSE_Blocks.CRACKED_DARK_GREEN_CONCRETE.asItem(), NSE_Blocks.CRACKED_ORANGE_CONCRETE.asItem()),
                "concrete_cracked");
        concreteRecipe(exporter, NSE_Blocks.CONCRETE, null);
        concreteRecipe(exporter, NSE_Blocks.GREEN_CONCRETE, Items.LIME_DYE);
        concreteRecipe(exporter, NSE_Blocks.DARK_GREEN_CONCRETE, Items.GREEN_DYE);
        concreteRecipe(exporter, NSE_Blocks.YELLOW_CONCRETE, Items.YELLOW_DYE);
        concreteRecipe(exporter, NSE_Blocks.ORANGE_CONCRETE, Items.ORANGE_DYE);
        concreteRecipe(exporter, NSE_Blocks.BLUE_CONCRETE, Items.BLUE_DYE);
        concreteRecipe(exporter, NSE_Blocks.RED_CONCRETE, Items.RED_DYE);
        concreteRecipe(exporter, NSE_Blocks.WHITE_CONCRETE, Items.WHITE_DYE);
        concreteRecipe(exporter, NSE_Blocks.BEIGE_CONCRETE, Items.LIGHT_GRAY_DYE);
        concreteWithBarsRecipe(exporter, NSE_Blocks.CONCRETE_WITH_BARS, NSE_Blocks.CRACKED_CONCRETE);
        concreteWithBarsRecipe(exporter, NSE_Blocks.DARK_GREEN_CONCRETE_WITH_BARS, NSE_Blocks.CRACKED_DARK_GREEN_CONCRETE);
        concreteWithBarsRecipe(exporter, NSE_Blocks.GREEN_CONCRETE_WITH_BARS, NSE_Blocks.CRACKED_GREEN_CONCRETE);
        concreteWithBarsRecipe(exporter, NSE_Blocks.ORANGE_CONCRETE_WITH_BARS, NSE_Blocks.CRACKED_ORANGE_CONCRETE);
        concreteWithBarsRecipe(exporter, NSE_Blocks.YELLOW_CONCRETE_WITH_BARS, NSE_Blocks.CRACKED_YELLOW_CONCRETE);
        concreteWithBarsRecipe(exporter, NSE_Blocks.BLUE_CONCRETE_WITH_BARS, NSE_Blocks.CRACKED_BLUE_CONCRETE);
        concreteWithBarsRecipe(exporter, NSE_Blocks.RED_CONCRETE_WITH_BARS, NSE_Blocks.CRACKED_RED_CONCRETE);
        concreteWithBarsRecipe(exporter, NSE_Blocks.WHITE_CONCRETE_WITH_BARS, NSE_Blocks.CRACKED_WHITE_CONCRETE);
        concreteWithBarsRecipe(exporter, NSE_Blocks.BEIGE_CONCRETE_WITH_BARS, NSE_Blocks.CRACKED_BEIGE_CONCRETE);

        wallpaperRecipe(exporter, NSE_Blocks.BEIGE_WALLPAPER, Items.WHITE_DYE);
        wallpaperRecipe(exporter, NSE_Blocks.GREEN_WALLPAPER, Items.GREEN_DYE);
        wallpaperRecipe(exporter, NSE_Blocks.BROWN_WALLPAPER, Items.BROWN_DYE);
        offerDyeableRecipes(exporter,
                List.of(Items.GREEN_DYE, Items.WHITE_DYE, Items.BROWN_DYE   ),
                List.of(NSE_Blocks.GREEN_WALLPAPER.asItem(), NSE_Blocks.BEIGE_WALLPAPER.asItem(), NSE_Blocks.BROWN_WALLPAPER.asItem()),
                "wallpaper");

        linoleumRecipe(exporter, NSE_Blocks.GREEN_LINOLEUM, Items.GREEN_DYE);
        linoleumRecipe(exporter, NSE_Blocks.BLUE_LINOLEUM, Items.BLUE_DYE);
        linoleumRecipe(exporter, NSE_Blocks.RED_LINOLEUM, Items.RED_DYE);
        linoleumRecipe(exporter, NSE_Blocks.GRAY_LINOLEUM, Items.GRAY_DYE);
        linoleumRecipe(exporter, NSE_Blocks.ORANGE_LINOLEUM, Items.GRAY_DYE);
        linoleumRecipe(exporter, NSE_Blocks.BROWN_LINOLEUM, Items.GRAY_DYE);
        linoleumRecipe(exporter, NSE_Blocks.CYAN_LINOLEUM, Items.GRAY_DYE);
        offerStonecuttingRecipe(exporter, RecipeCategory.BUILDING_BLOCKS, NSE_Blocks.CROSS_ORANGE_LINOLEUM, NSE_Blocks.ORANGE_LINOLEUM);
        offerStonecuttingRecipe(exporter, RecipeCategory.BUILDING_BLOCKS, NSE_Blocks.CROSS_BROWN_LINOLEUM, NSE_Blocks.ORANGE_LINOLEUM);
        offerDyeableRecipes(exporter,
                List.of(Items.GREEN_DYE, Items.ORANGE_DYE, Items.BLUE_DYE, Items.RED_DYE, Items.GRAY_DYE, Items.BROWN_DYE, Items.CYAN_DYE),
                List.of(NSE_Blocks.GREEN_LINOLEUM.asItem(), NSE_Blocks.ORANGE_LINOLEUM.asItem(), NSE_Blocks.BLUE_LINOLEUM.asItem(),
                        NSE_Blocks.RED_LINOLEUM.asItem(), NSE_Blocks.GRAY_LINOLEUM.asItem(), NSE_Blocks.BROWN_LINOLEUM.asItem(), NSE_Blocks.CYAN_LINOLEUM.asItem()),
                "linoleum");

        ShapedRecipeJsonBuilder.create(RecipeCategory.TOOLS, NSE_Items.SCREWDRIVER, 1)
                .input('I', Items.IRON_INGOT).input('S', Items.STICK)
                .pattern("I").pattern("S")
                .criterion(hasItem(Items.IRON_INGOT), conditionsFromItem(Items.IRON_INGOT))
                .criterion(hasItem(Items.STICK), conditionsFromItem(Items.STICK))
                .offerTo(exporter);

        ShapelessRecipeJsonBuilder.create(RecipeCategory.MISC, NSE_Items.SICKLE_PATTERN).input(Items.PAPER).input(NSE_Items.SICKLE)
                .criterion(FabricRecipeProvider.hasItem(Items.PAPER), FabricRecipeProvider.conditionsFromItem(NSE_Items.SICKLE_PATTERN))
                .criterion(FabricRecipeProvider.hasItem(NSE_Items.SICKLE), FabricRecipeProvider.conditionsFromItem(NSE_Items.SICKLE_PATTERN))
                .offerTo(exporter);
        ShapelessRecipeJsonBuilder.create(RecipeCategory.MISC, NSE_Items.STAR_PATTERN).input(Items.PAPER).input(NSE_Items.SICKLE).input(Items.RED_DYE)
                .criterion(FabricRecipeProvider.hasItem(Items.PAPER), FabricRecipeProvider.conditionsFromItem(NSE_Items.STAR_PATTERN))
                .criterion(FabricRecipeProvider.hasItem(NSE_Items.SICKLE), FabricRecipeProvider.conditionsFromItem(NSE_Items.STAR_PATTERN))
                .criterion(FabricRecipeProvider.hasItem(Items.RED_DYE), FabricRecipeProvider.conditionsFromItem(NSE_Items.STAR_PATTERN))
                .offerTo(exporter);
        ShapelessRecipeJsonBuilder.create(RecipeCategory.MISC, NSE_Items.GRAIN_PATTERN).input(Items.PAPER).input(Items.WHEAT)
                .criterion(FabricRecipeProvider.hasItem(Items.PAPER), FabricRecipeProvider.conditionsFromItem(NSE_Items.GRAIN_PATTERN))
                .criterion(FabricRecipeProvider.hasItem(Items.WHEAT), FabricRecipeProvider.conditionsFromItem(NSE_Items.GRAIN_PATTERN))
                .offerTo(exporter);
    }
}